/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

/*
** Generated by X-Designer
*/
#ifndef _UITemplateDrawer_h
#define _UITemplateDrawer_h

#define XD_MOTIF

#include <xdclass.h>

class UITemplateDrawer_c: public xd_XmForm_c {
public:
	virtual void create (Widget parent, char *widget_name = NULL);
protected:
	Widget UITemplateDrawer;
	Widget drag_;
	Widget menu1_;
	Widget merge_;
	Widget replace_;
	Widget menu2_;
public:
	static void openFolderCB( Widget, XtPointer, XtPointer );
	virtual void openFolderCB( Widget, XtPointer ) = 0;
	static void replaceCB( Widget, XtPointer, XtPointer );
	virtual void replaceCB( Widget, XtPointer ) = 0;
	static void mergeCB( Widget, XtPointer, XtPointer );
	virtual void mergeCB( Widget, XtPointer ) = 0;
	static void editCB( Widget, XtPointer, XtPointer );
	virtual void editCB( Widget, XtPointer ) = 0;
	static void makeCB( Widget, XtPointer, XtPointer );
	virtual void makeCB( Widget, XtPointer ) = 0;
};

typedef UITemplateDrawer_c *UITemplateDrawer_p;


extern UITemplateDrawer_p UITemplateDrawer;


#endif
