#ifndef mcarousel_DEFINED
#define mcarousel_DEFINED

#include <MMotifOnly.hpp>
#include <Carousel.h>

class MCarousel : public MDrawingArea
{
public:
	void    init(MWidget *parent, const char *name, ...);
};

#endif
