/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

// File VariableTask.h
// Baudouin Raoult - ECMWF Jul 99

#ifndef VariableTask_H
#define VariableTask_H

#ifndef Task_H
#include "Task.h"
#endif


#ifndef ReplyObserver_H
#include "ReplyObserver.h"
#endif

#include "TaskObserver.h"
#include "IconObject.h"
#include "Dependancy.h"
#include "Request.h"


class VariableEditor;


class VariableTask : public Task, public ReplyObserver {
public:

// -- Exceptions
	// None

// -- Contructors

	VariableTask(VariableEditor&, const string&,const Request&);

// -- Destructor

	virtual ~VariableTask(); // Change to virtual if base class

// -- Convertors
	// None

// -- Operators


// -- Methods
	// None

// -- Overridden methods
	// None

// -- Class members
	// None

// -- Class methods
	// None

	// Uncomment for persistent, remove otherwise
	// static os_typespec* get_os_typespec();

protected:

// -- Members
	// None

// -- Methods
	
	 void print(ostream&) const; // Change to virtual if base class	

// -- Overridden methods
	// None

// -- Class members
	// None

// -- Class methods
	// None

private:

// No copy allowed

	VariableTask(const VariableTask&);
	VariableTask& operator=(const VariableTask&);

// -- Members

	VariableEditor& editor_;
	string          service_;
	Request         request_;
	IconClass*      class_;

// -- Methods


// -- Overridden methods

	// From Task
	virtual void start();
	// From ReplyObserver
	virtual void reply(const Request&, int);
	virtual void progress(const Request&);  
	virtual void message(const string&);
// -- Class members
	// None

// -- Class methods
	// None

// -- Friends

	//friend ostream& operator<<(ostream& s,const VariableTask& p)
	//	{ p.print(s); return s; }

};

inline void destroy(VariableTask**) {}

// If persistent, uncomment, otherwise remove
//#ifdef _ODI_OSSG_
//OS_MARK_SCHEMA_TYPE(VariableTask);
//#endif

#endif
