/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

// File Tools.h
// Baudouin Raoult - ECMWF Jul 99

#ifndef Tools_H
#define Tools_H

// Headers
// #ifndef   machine_H
// #include <machine.h>
// #endif

// Forward declarations

// class ostream;
// typedef class _Pvts os_typespec; // Remove if not persistant

// 

#include "Metview.h"
#include <Xm/Xm.h>


class Tools {
public:

// -- Exceptions
	// None

// -- Contructors

	Tools(request*);

// -- Destructor

	~Tools(); // Change to virtual if base class

// -- Convertors
	// None

// -- Operators
	// None

// -- Methods
	void start();


// -- Overridden methods
	// None

// -- Class members
	// None

// -- Class methods

	static void load(request*);
	static void make(Widget);
	

	// Uncomment for persistent, remove otherwise
	// static os_typespec* get_os_typespec();

protected:

// -- Members
	// None

// -- Methods
	
	// void print(ostream&) const; // Change to virtual if base class	

// -- Overridden methods
	// None

// -- Class members
	// None

// -- Class methods
	// None

private:

// No copy allowed

	Tools(const Tools&);
	Tools& operator=(const Tools&);

// -- Members

	request*    request_;

// -- Methods
	// None

// -- Overridden methods
	// None

// -- Class members
	// None

// -- Class methods
	// None

// -- Friends

	//friend ostream& operator<<(ostream& s,const Tools& p)
	//	{ p.print(s); return s; }

};

inline void destroy(Tools**) {}

// If persistent, uncomment, otherwise remove
//#ifdef _ODI_OSSG_
//OS_MARK_SCHEMA_TYPE(Tools);
//#endif

#endif
