/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/


#ifndef  SYMBOLS_H
#define  SYMBOLS_H

extern WidgetClass symbolsWidgetClass;
typedef struct _SymbolsClassRec * SymbolsWidgetClass;
typedef struct _SymbolsRec      * SymbolsWidget;

#if defined(__cplusplus) || defined(c_plusplus)
extern "C" {
#endif

extern Widget CreateSymbolsWidget(Widget,char*,Arg*,int);
extern void SymbolsSetPixmaps(Widget,int,Pixmap*);

#if defined(__cplusplus) || defined(c_plusplus)
}
#endif


#endif
