/***************************** LICENSE START ***********************************

 Copyright 2014 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

#ifndef MetviewService_H
#define MetviewService_H

#include "Service.h"

class Task;
class IconObject;

class MetviewService : public Service 
{
public:
	MetviewService(const string&);
	virtual ~MetviewService(); 

private:
	// No copy allowed
	MetviewService(const MetviewService&);
	MetviewService& operator=(const MetviewService&);

	virtual Task* task(const Action&,IconObject*);

};

inline void destroy(MetviewService**) {}

#endif
