/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

// File Ask.h
// Baudouin Raoult - ECMWF Nov 98

#ifndef Ask_H
#define Ask_H

#include <string>
using std::string;

#include "UIAsk.h"
#include "Dialog.h"

class Ask : public Dialog<Ask,UIAsk_c> {
public:

// -- Exceptions
	// None

// -- Contructors

	Ask();

// -- Destructor

	~Ask(); // Change to virtual if base class

// -- Convertors
	// None

// -- Operators
	// None

// -- Methods
	// None

// -- Overridden methods
	// None

// -- Class members
	// None

// -- Class methods

	static bool show(string&,const char*,...);

	// Uncomment for persistent, remove otherwise
	// static os_typespec* get_os_typespec();

protected:

// -- Members
	// None

// -- Methods
	
	// void print(ostream&) const; // Change to virtual if base class	

// -- Overridden methods
	// None

// -- Class members
	// None

// -- Class methods
	// None

private:

// No copy allowed

	Ask(const Ask&);
	Ask& operator=(const Ask&);

// -- Methods

	bool show(const char*,string&);

// -- Overridden methods


// -- Class members


// -- Class methods
	// None

// -- Friends

	//friend ostream& operator<<(ostream& s,const Ask& p)
	//	{ p.print(s); return s; }

};

inline void destroy(Ask**) {}

// If persistent, uncomment, otherwise remove
//#ifdef _ODI_OSSG_
//OS_MARK_SCHEMA_TYPE(Ask);
//#endif

#endif
