/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

/*
** Generated by X-Designer
*/
#ifndef _UIEditor_h
#define _UIEditor_h

#define XD_MOTIF

#include <xdclass.h>

class UIEditor_c: public xd_ApplicationShell_c {
public:
	virtual void create (Display *display, char *app_name, int app_argc, char **app_argv, char *app_class_name = NULL);
protected:
	Widget UIEditor;
	Widget form_;
	Widget drawer_;
	Widget main_;
	Widget icon_;
	Widget name_;
	Widget stay_;
	Widget tool_;
public:
	static void textEditorCB( Widget, XtPointer, XtPointer );
	virtual void textEditorCB( Widget, XtPointer ) = 0;
	static void resetCB( Widget, XtPointer, XtPointer );
	virtual void resetCB( Widget, XtPointer ) = 0;
	static void applyCB( Widget, XtPointer, XtPointer );
	virtual void applyCB( Widget, XtPointer ) = 0;
	static void closeCB( Widget, XtPointer, XtPointer );
	virtual void closeCB( Widget, XtPointer ) = 0;
};

typedef UIEditor_c *UIEditor_p;


extern UIEditor_p UIEditor;


#endif
