/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

#ifndef NoEditor_H
#include "NoEditor.h"
#endif

#ifndef EditorFactory_H
#include "EditorFactory.h"
#endif


NoEditor::NoEditor(const IconClass& name,const string& kind):
	XEditor<UINoEditor_c>(name,kind)
{
}

NoEditor::~NoEditor()
{
}

void NoEditor::apply()
{
}

void NoEditor::reset()
{
}

void NoEditor::close()
{
}

void NoEditor::merge(IconObject*)
{
}
void NoEditor::replace(IconObject*)
{
}

string NoEditor::alternateEditor()
{
	return "NoEditor";
}


static EditorMaker<NoEditor> editorMaker("NoEditor");

