#ifndef mSymbols_DEFINED
#define mSymbols_DEFINED

#include <MMotifOnly.hpp>
#include <Symbols.h>

class MSymbolsWidget : public MDrawingArea
{
public:
	void    init(MWidget *parent, const char *name, ...);
	void setPixmaps(int n,Pixmap *p) { SymbolsSetPixmaps(widget,n,p); }
};

#endif
