/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

// File Dropping
// Sylvie Thepaut - ECMWF Oct 99

#ifndef Dropping_H
#define Dropping_H

class IconObject;
class DropSite;

class Dropping {
public:

// -- Exceptions
	// None

// -- Contructors

	Dropping();

// -- Destructor

	virtual ~Dropping(); // Change to virtual if base class

// -- Convertors
	// None

// -- Operators
	// None

// -- Methods

	virtual IconObject* object(DropSite&) = 0;
	virtual bool copy() = 0;


	int x();
	int y();
	void position(int,int);


// -- Overridden methods
	// None

// -- Class members
	// None

// -- Class methods
	// None

	// Uncomment for persistent, remove otherwise
	// static os_typespec* get_os_typespec();

protected:

// -- Members
	// None

// -- Methods
	
	// void print(ostream&) const; // Change to virtual if base class	

// -- Overridden methods
	// None

// -- Class members
	// None

// -- Class methods
	// None

private:

// No copy allowed

	Dropping(const Dropping&);
	Dropping& operator=(const Dropping&);

// -- Members

	int x_;
	int y_;

// -- Methods
	// None

// -- Overridden methods
	// None

// -- Class members
	// None

// -- Class methods
	// None

// -- Friends

	//friend ostream& operator<<(ostream& s,const Dropping& p)
	//	{ p.print(s); return s; }

};

inline void destroy(Dropping**) {}

// If persistent, uncomment, otherwise remove
//#ifdef _ODI_OSSG_
//OS_MARK_SCHEMA_TYPE(Dropping);
//#endif

#endif
