/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

// File ResolveTransaction.h
// Baudouin Raoult - ECMWF Jul 99

#ifndef ResolveTransaction_H
#define ResolveTransaction_H

// Headers
// #ifndef   machine_H
// #include <machine.h>
// #endif
#ifndef  Transaction_H
#include <Transaction.h>
#endif
#ifndef  TaskObserver_H
#include <TaskObserver.h>
#endif

// Forward declarations

// class ostream;
// typedef class _Pvts os_typespec; // Remove if not persistant

// 

class ResolveTransaction : public Transaction, public TaskObserver {
public:

// -- Exceptions
	// None

// -- Contructors

	ResolveTransaction();

// -- Destructor

	~ResolveTransaction(); // Change to virtual if base class

// -- Convertors
	// None

// -- Operators
	// None

// -- Methods
	// None

// -- Overridden methods
	// None

// -- Class members
	// None

// -- Class methods

	static void init();

	// Uncomment for persistent, remove otherwise
	// static os_typespec* get_os_typespec();

protected:

// -- Members
	// None

// -- Methods
	
	// void print(ostream&) const; // Change to virtual if base class	

// -- Overridden methods
	// None

// -- Class members
	// None

// -- Class methods
	// None

private:

// No copy allowed

	ResolveTransaction(const ResolveTransaction&);
	ResolveTransaction(MvTransaction*);
	ResolveTransaction& operator=(const ResolveTransaction&);

// -- Members
	// None

// -- Methods
	// None

// -- Overridden methods
	// -- From MvTransaction

	virtual MvTransaction *cloneSelf();
	virtual void callback(MvRequest&); 
	// -- From TaskObserver

	virtual void success(Task*,const Request&);
	void failure(Task*);



// -- Class members
	// None

// -- Class methods
	// None

// -- Friends

	//friend ostream& operator<<(ostream& s,const ResolveTransaction& p)
	//	{ p.print(s); return s; }

};

inline void destroy(ResolveTransaction**) {}

// If persistent, uncomment, otherwise remove
//#ifdef _ODI_OSSG_
//OS_MARK_SCHEMA_TYPE(ResolveTransaction);
//#endif

#endif
