#ifndef micon_DEFINED
#define micon_DEFINED

#include <MMotifOnly.hpp>
#include <IconWidget.h>

class MIconLabel : public MWidget
{
public:
        void    init(MWidget *parent, char *name, ...);
};
	
class MIconButton : public MWidget
{
public:
        void    init(MWidget *parent, char *name, ...);
};

#endif
