/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

// .NAME:
//  OutputFormatAction
//
// .AUTHOR:
//  Fernando Ii
//
// .SUMMARY:
//  Describes the OutputFormatAction class, a derived class
//  for the various types of requests in uPlot which requires
//  saving all the output formats related to this plot.
//
//
// .CLIENTS:
//  PlotModAction class, called by uPlot main module
//
// .RESPONSABILITY:
//  This class will provide all support for the visualisation
//  actions whithin uPlot
//
// .COLLABORATORS:
//  Builder, Matcher, Drawer
//
// .ASCENDENT:
//  PlotModAction
//
// .DESCENDENT:
//
//
#ifndef OutputFormatAction_H
#define OutputFormatAction_H

#include "PlotModAction.h"

class OutputFormatAction: public PlotModAction
{

public:

  // Constructors
  OutputFormatAction(const Cached name): PlotModAction(name) {}

  // Destructor
  ~OutputFormatAction() {}

  // Methods
  // Overriden from PlotModAction
  virtual void Execute ( PmContext& context );
};

#endif
