/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

// File ExternalServiceEditor.h
// Fernando Ii - ECMWF Nov 2009

#ifndef ExternalServiceEditor_H
#define ExternalServiceEditor_H

#ifndef ExternalEditor_H
#include "ExternalEditor.h"
#endif


class ExternalServiceEditor : public ExternalEditor {
public:

	// -- Contructors
	ExternalServiceEditor(const IconClass&,const string&);

	// -- Destructor
	virtual ~ExternalServiceEditor() {} // Change to virtual if base class

private:

	// No copy allowed
	ExternalServiceEditor(const ExternalServiceEditor&);
	ExternalServiceEditor& operator=(const ExternalServiceEditor&);

	// From ExternalEditor
	virtual void edit();
	virtual void raise();
	virtual string command() {}

};

inline void destroy(ExternalServiceEditor**) {}

// If persistent, uncomment, otherwise remove
//#ifdef _ODI_OSSG_
//OS_MARK_SCHEMA_TYPE(ExternalTextEditor);
//#endif

#endif
