#ifndef mColors_DEFINED
#define mColors_DEFINED

#include <MMotifOnly.hpp>
#include <Colors.h>

class MColorsWidget : public MDrawingArea
{
public:
	void    init(MWidget *parent, const char *name, ...);
	void setPixels(int n,Pixel *p) { ColorsSetPixels(widget,n,p); }
};

#endif
