/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

// File IconInfo.h
// Baudouin Raoult - ECMWF Jul 99

#ifndef IconInfo_H
#define IconInfo_H

#include "Request.h"

class IconInfo {
public:

// -- Exceptions
	// None

// -- Contructors

	IconInfo(const Request&);

// -- Destructor

	~IconInfo(); // Change to virtual if base class

// -- Convertors
	// None

// -- Operators
	// None

// -- Methods

	int x() const;
	int y() const;

	void position(int,int);

	void save(const Path&);

	bool fix(const string&);

// -- Overridden methods
	// None

// -- Class members
	// None

// -- Class methods
	// None

	// Uncomment for persistent, remove otherwise
	// static os_typespec* get_os_typespec();

protected:

// -- Members
	// None

// -- Methods
	
	// void print(ostream&) const; // Change to virtual if base class	

// -- Overridden methods
	// None

// -- Class members
	// None

// -- Class methods
	// None

private:

// No copy allowed

	IconInfo(const IconInfo&);
	IconInfo& operator=(const IconInfo&);

// -- Members

	bool    modified_;
	Request info_;

// -- Methods
	// None

// -- Overridden methods
	// None

// -- Class members
	// None

// -- Class methods
	// None

// -- Friends

	//friend ostream& operator<<(ostream& s,const IconInfo& p)
	//	{ p.print(s); return s; }

};

inline void destroy(IconInfo**) {}

// If persistent, uncomment, otherwise remove
//#ifdef _ODI_OSSG_
//OS_MARK_SCHEMA_TYPE(IconInfo);
//#endif

#endif
