/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

#ifndef MvQHighlighter_H
#define MvQHighlighter_H

#include <QSyntaxHighlighter>

#include <QHash>
#include <QTextCharFormat>

class QTextDocument;

class MvQHighlighter : public QSyntaxHighlighter
 {
 public:
    MvQHighlighter(QTextDocument *parent = 0);

 protected:
     void highlightBlock(const QString &text);

 private:
     struct HighlightingRule
     {
         QRegExp pattern;
         QTextCharFormat format;
     };

     QVector<HighlightingRule> highlightingRules;
 };


class MvQWmsGetCapHighlighter : public QSyntaxHighlighter
{
public:
     MvQWmsGetCapHighlighter(QTextDocument *parent = 0);
    ~MvQWmsGetCapHighlighter(){}

     void highlightBlock(const QString &text);

protected:

     QRegExp commentStartExpression;
     QRegExp commentEndExpression;

     QTextCharFormat tagNameFormat;
     QTextCharFormat attributeNameFormat;
     QTextCharFormat tagSymbolsFormat;
     QTextCharFormat quotationFormat;
     QTextCharFormat multiLineCommentFormat;

     struct HighlightingRule
     {
         QRegExp pattern;
         QTextCharFormat format;
     };
     QVector<HighlightingRule> highlightingRules;
};

#endif
