/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

// File TempIconClass.h
// Baudouin Raoult - ECMWF Jul 99

#ifndef TempIconClass_H
#define TempIconClass_H

#ifndef Metview_H
#include "Metview.h"
#endif

#ifndef IconClass_H
#include "IconClass.h"
#endif

#ifndef Language_H
#include "Language.h"
#endif

#ifndef Basic_H
#include "Basic.h"
#endif

#ifndef IconFactory_H
#include "IconFactory.h"
#endif
class TempIconClass : public IconClass {
public:

// -- Exceptions
	// None

// -- Contructors

	TempIconClass(const Request& lang,const IconClass* = 0);
	
// -- Destructor

	virtual ~TempIconClass(); // Change to virtual if base class

// -- Convertors
	// None

// -- Operators
	// None

// -- Methods
	

// -- Overridden methods
	// None

// -- Class members



// -- Class methods
	// None

	// Uncomment for persistent, remove otherwise
	// static os_typespec* get_os_typespec();

protected:

// -- Members
	// None

// -- Methods
	
	// void print(ostream&) const; // Change to virtual if base class	

// -- Overridden methods
	// None

// -- Class members
	// None

// -- Class methods
	// None

private:

// No copy allowed

	TempIconClass(const TempIconClass&);
	TempIconClass& operator=(const TempIconClass&);

// -- Members

	Language         lang_;
	IconMaker<Basic> maker_;

// -- Methods
	// None

// -- Overridden methods
	// None

	virtual Language& language() const;
	virtual string editor() const;

// -- Class members
	// None

// -- Class methods
	// None

// -- Friends

	//friend ostream& operator<<(ostream& s,const TempIconClass& p)
	//	{ p.print(s); return s; }

};

inline void destroy(TempIconClass**) {}

// If persistent, uncomment, otherwise remove
//#ifdef _ODI_OSSG_
//OS_MARK_SCHEMA_TYPE(TempIconClass);
//#endif

#endif
