/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

// File IconSelection.h
// Baudouin Raoult - ECMWF Nov 98

#ifndef IconSelection_H
#define IconSelection_H

#include <string>
using std::string;

#include "UIIconSelection.h"
#include "Dialog.h"

class Folder;
class IconObject;
struct request;

class IconSelection : public Dialog<IconSelection,UIIconSelection_c> {
public:

// -- Exceptions
	// None

// -- Contructors

	IconSelection();

// -- Destructor

	~IconSelection(); // Change to virtual if base class

// -- Convertors
	// None

// -- Operators
	// None

// -- Methods
	// None

// -- Overridden methods
	// None

// -- Class members
	// None

// -- Class methods

	static IconObject* make(Folder*,int x = 0,int y = 0);
	static void load(request*);


	// Uncomment for persistent, remove otherwise
	// static os_typespec* get_os_typespec();

protected:

// -- Members
	// None

// -- Methods
	
	// void print(ostream&) const; // Change to virtual if base class	

// -- Overridden methods
	// None

// -- Class members
	// None

// -- Class methods
	// None

private:

// No copy allowed

	IconSelection(const IconSelection&);
	IconSelection& operator=(const IconSelection&);

// -- Members

	string name_;

// -- Methods


	IconObject* choose(Folder*,int,int);


// -- Overridden methods

	virtual void okCB( Widget, XtPointer ) ;

// -- Class members


// -- Class methods
	// None

// -- Friends

	//friend ostream& operator<<(ostream& s,const IconSelection& p)
	//	{ p.print(s); return s; }

};

inline void destroy(IconSelection**) {}

// If persistent, uncomment, otherwise remove
//#ifdef _ODI_OSSG_
//OS_MARK_SCHEMA_TYPE(IconSelection);
//#endif

#endif
