/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

/*
** Generated by X-Designer
*/
#ifndef _UILine_h
#define _UILine_h

#define XD_MOTIF

#include <xdclass.h>

class UILine_c: public xd_XmForm_c {
public:
	virtual void create (Widget parent, char *widget_name = NULL);
protected:
	Widget UILine;
	Widget name_;
	Widget help_arrow_;
	Widget form_;
	Widget help_button_;
	Widget where_;
	Widget helper_;
public:
	static void helpCB( Widget, XtPointer, XtPointer );
	virtual void helpCB( Widget, XtPointer ) = 0;
	static void comboCB( Widget, XtPointer, XtPointer );
	virtual void comboCB( Widget, XtPointer ) = 0;
	static void setCB( Widget, XtPointer, XtPointer );
	virtual void setCB( Widget, XtPointer ) = 0;
};

typedef UILine_c *UILine_p;


extern UILine_p UILine;


#endif
