/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

// File ColorLine.h
// Baudouin Raoult - ECMWF Jul 99

#ifndef ColorLine_H
#define ColorLine_H

#include "UIColorLine.h"
#include "RequestPanelLine.h"

class ColorLine : public XRequestPanelLine<UIColorLine_c> {
public:

// -- Contructors

	ColorLine(RequestPanel&,const Parameter&);

// -- Destructor

	virtual ~ColorLine(); 

private:

// No copy allowed

	ColorLine(const ColorLine&);
	ColorLine& operator=(const ColorLine&);

// -- Members
	
// -- Methods

// -- Overridden methods

	// From RequestPanelItem

       virtual void refresh(const vector<string>&);

};

#endif
