/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

// File TextEditor.h
// Baudouin Raoult - ECMWF Jul 99

#ifndef TextEditor_H
#define TextEditor_H

#ifndef BaseTextEditor_H
#include "BaseTextEditor.h"
#endif

class TextEditor : public BaseTextEditor {
public:

// -- Exceptions
	// None

// -- Contructors

	TextEditor(const IconClass&,const string&);

// -- Destructor

	virtual ~TextEditor(); // Change to virtual if base class

// -- Convertors
	// None

// -- Operators
	// None

// -- Methods
	// None

// -- Overridden methods
	// None

// -- Class members
	// None

// -- Class methods
	// None

	// Uncomment for persistent, remove otherwise
	// static os_typespec* get_os_typespec();

protected:

// -- Members
// -- Methods
	
	// void print(ostream&) const; // Change to virtual if base class	

// -- Overridden methods

// -- Class members
	// None

// -- Class methods
	// None

private:

// No copy allowed

	TextEditor(const TextEditor&);
	TextEditor& operator=(const TextEditor&);

// -- Members


// -- Methods
	// None

// -- Overridden methods

// -- Class members
	// None

// -- Class methods
	// None

// -- Friends

	//friend ostream& operator<<(ostream& s,const TextEditor& p)
	//	{ p.print(s); return s; }

};

inline void destroy(TextEditor**) {}

// If persistent, uncomment, otherwise remove
//#ifdef _ODI_OSSG_
//OS_MARK_SCHEMA_TYPE(TextEditor);
//#endif

#endif
