/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

// File Recents.h
// Baudouin Raoult - ECMWF Oct 99

#ifndef Recents_H
#define Recents_H

#include <string>
using std::string;

#include <Xm/Xm.h>

class Folder;

class Recents {
public:

// -- Exceptions
	// None

// -- Contructors

	Recents();

// -- Destructor

	~Recents(); // Change to virtual if base class

// -- Convertors
	// None

// -- Operators
	// None

// -- Methods

	void install(Widget);
	void top(Folder*);
	void open(const char*);

// -- Overridden methods
	// None

// -- Class members
	// None

// -- Class methods
	// None

	// Uncomment for persistent, remove otherwise
	// static os_typespec* get_os_typespec();

protected:

// -- Members
	// None

// -- Methods
	
	// void print(ostream&) const; // Change to virtual if base class	

// -- Overridden methods
	// None

// -- Class members
	// None

// -- Class methods
	// None

private:

// No copy allowed

	Recents(const Recents&);
	Recents& operator=(const Recents&);

// -- Members
	
	int    fixed_;
	Widget widget_;
	string name_;

// -- Methods
	
	Widget add(const string&);
	void top(const string&);
	void remove(const string&);
	void rename(const string&,const string&);


// -- Overridden methods


// -- Class members
	// None

// -- Class methods
	// None

// -- Friends

	//friend ostream& operator<<(ostream& s,const Recents& p)
	//	{ p.print(s); return s; }

};

inline void destroy(Recents**) {}

// If persistent, uncomment, otherwise remove
//#ifdef _ODI_OSSG_
//OS_MARK_SCHEMA_TYPE(Recents);
//#endif

#endif
