#ifndef mTab_DEFINED
#define mTab_DEFINED

#include <MMotifOnly.hpp>
#include <Tab.h>

class MTabWidget : public MDrawingArea
{
public:
	void    init(MWidget *parent, const char *name, ...);
	void open();
	void close();

	Boolean closed();

	void set(MWidget&);
};

#endif
