/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

/*
** Generated by X-Designer
*/
#ifndef _UIColorName_h
#define _UIColorName_h

#define XD_MOTIF

#include <xdclass.h>

class UIColorName_c: public xd_XmForm_c {
public:
	virtual void create (Widget parent, char *widget_name = NULL);
protected:
	Widget UIColorName;
	Widget colour_;
	Widget name_;
public:
	static void activateCB( Widget, XtPointer, XtPointer );
	virtual void activateCB( Widget, XtPointer ) = 0;
};

typedef UIColorName_c *UIColorName_p;


extern UIColorName_p UIColorName;


#endif
