/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

/*
** Generated by X-Designer
*/
/*
**LIBS: -lXm -lXt -lX11
*/

#include <stdlib.h>
#include <X11/Xatom.h>
#include <X11/Intrinsic.h>
#include <X11/Shell.h>

#include <Xm/Xm.h>
#include <Xm/DialogS.h>
#include <Xm/Form.h>
#include <Xm/Scale.h>

#include <xdclass.h>

class UIScaleLine_c: public xd_XmForm_c {
public:
	virtual void create (Widget parent, char *widget_name = NULL);
protected:
	Widget UIScaleLine;
	Widget scale_;
public:
	static void changedCB( Widget, XtPointer, XtPointer );
	virtual void changedCB( Widget, XtPointer ) = 0;
};

typedef UIScaleLine_c *UIScaleLine_p;

UIScaleLine_p UIScaleLine = (UIScaleLine_p) NULL;



void UIScaleLine_c::create (Widget parent, char *widget_name)
{
	Widget children[1];      /* Children to manage */
	Arg al[64];                    /* Arg List */
	register int ac = 0;           /* Arg Count */

	if ( !widget_name )
		widget_name = "UIScaleLine";

	XtSetArg(al[ac], XmNautoUnmanage, FALSE); ac++;
	UIScaleLine = XmCreateForm ( parent, widget_name, al, ac );
	ac = 0;
	_xd_rootwidget = UIScaleLine;
	XtSetArg(al[ac], XmNshowValue, TRUE); ac++;
	XtSetArg(al[ac], XmNorientation, XmHORIZONTAL); ac++;
	XtSetArg(al[ac], XmNprocessingDirection, XmMAX_ON_RIGHT); ac++;
	scale_ = XmCreateScale ( UIScaleLine, "scale_", al, ac );
	ac = 0;

	XtSetArg(al[ac], XmNtopAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNleftAttachment, XmATTACH_FORM); ac++;
	XtSetValues ( scale_,al, ac );
	ac = 0;
	XtAddCallback (scale_, XmNvalueChangedCallback, changedCB, (XtPointer) this);
	children[ac++] = scale_;
	XtManageChildren(children, ac);
	ac = 0;
}

void UIScaleLine_c::changedCB( Widget widget, XtPointer client_data, XtPointer call_data )
{
	UIScaleLine_p instance = (UIScaleLine_p) client_data;
	instance->changedCB ( widget, call_data );
}

