/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

/*
 * Copyright 1991 Addison-Wesley Publishing Company
 *                (see the file "COPYRIGHTS" for details)
 */

#ifndef _IconWidget_h
#define _IconWidget_h

#ifndef NeedFunctionPrototypes
#define NeedFunctionPrototypes 0
#endif

#include <Xm/Xm.h>

#define XmNiconOffset     "iconOffset"
#define XmCIconOffset     "IconOffset"

extern WidgetClass iconWidgetClass;

typedef struct _IconWidgetClassRec * IconWidgetClass;
typedef struct _IconWidgetRec      * IconWidget;

Widget CreateIconLabel (
#if NeedFunctionPrototypes
    Widget       /* parent   */,
    char*        /* name     */,
    ArgList      /* arglist  */,
    Cardinal     /* argcount */,
    char*        /* pixmap   */
#endif
);

Widget CreateIconButton (
#if NeedFunctionPrototypes
    Widget       /* parent   */,
    char*        /* name     */,
    ArgList      /* arglist  */,
    Cardinal     /* argcount */,
    char*        /* pixmap   */
#endif
);

#endif /* _IconWidget_h */
/* DON'T ADD ANYTHING AFTER THIS #endif */
