/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

/*
** Generated by X-Designer
*/
#ifndef _UIPaperDrawer_h
#define _UIPaperDrawer_h

#define XD_MOTIF

#include <xdclass.h>

class UIPaperDrawer_c: public xd_XmForm_c {
public:
	virtual void create (Widget parent, char *widget_name = NULL);
protected:
	Widget UIPaperDrawer;
	Widget height_;
	Widget width_;
public:
	static void a3CB( Widget, XtPointer, XtPointer );
	virtual void a3CB( Widget, XtPointer ) = 0;
	static void a4CB( Widget, XtPointer, XtPointer );
	virtual void a4CB( Widget, XtPointer ) = 0;
	static void rotateCB( Widget, XtPointer, XtPointer );
	virtual void rotateCB( Widget, XtPointer ) = 0;
	static void setCB( Widget, XtPointer, XtPointer );
	virtual void setCB( Widget, XtPointer ) = 0;
};

typedef UIPaperDrawer_c *UIPaperDrawer_p;


extern UIPaperDrawer_p UIPaperDrawer;


#endif
