/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

/*
** Generated by X-Designer
*/
#ifndef _UIScriptHelp_h
#define _UIScriptHelp_h

#define XD_MOTIF

#include <xdclass.h>

class UIScriptHelp_c: public xd_ApplicationShell_c {
public:
	virtual void create (Display *display, char *app_name, int app_argc, char **app_argv, char *app_class_name = NULL);
protected:
	Widget UIScriptHelp;
	Widget form_;
	Widget text_;
	Widget progressLabel_;
public:
	static void closeCB( Widget, XtPointer, XtPointer );
	virtual void closeCB( Widget, XtPointer ) = 0;
};

typedef UIScriptHelp_c *UIScriptHelp_p;


extern UIScriptHelp_p UIScriptHelp;


#endif
