/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

// File ODBFunctionsDrawer.h
// Baudouin Raoult - ECMWF Jul 99

#ifndef ODBFunctionsDrawer_H
#define ODBFunctionsDrawer_H

#ifndef EditorDrawer_H
#include "EditorDrawer.h"
#endif

#include "UIODBFunctionsDrawer.h"

class ODBEditor;

class ODBFunctionsDrawer : public XEditorDrawer<UIODBFunctionsDrawer_c> {
public:

// -- Exceptions
	// None

// -- Contructors

	ODBFunctionsDrawer(ODBEditor&);

// -- Destructor

	virtual ~ODBFunctionsDrawer(); // Change to virtual if base class

// -- Convertors
	// None

// -- Operators
	// None

// -- Methods

// -- Overridden methods
	// None

// -- Class members
	// None

// -- Class methods
	// None

	// Uncomment for persistent, remove otherwise
	// static os_typespec* get_os_typespec();

protected:

// -- Members
	// None

// -- Methods
	
	// void print(ostream&) const; // Change to virtual if base class	

// -- Overridden methods
	// None

// -- Class members
	// None

// -- Class methods
	// None

private:

// No copy allowed

	ODBFunctionsDrawer(const ODBFunctionsDrawer&);
	ODBFunctionsDrawer& operator=(const ODBFunctionsDrawer&);

// -- Members

	ODBEditor& editor_;

// -- Methods
	// None

// -- Overridden methods

	virtual void doubleClickCB(Widget,XtPointer);
	virtual void browseCB(Widget,XtPointer);

// -- Class members
	// None

// -- Class methods
	// None

// -- Friends

	//friend ostream& operator<<(ostream& s,const ODBFunctionsDrawer& p)
	//	{ p.print(s); return s; }

};

inline void destroy(ODBFunctionsDrawer**) {}

// If persistent, uncomment, otherwise remove
//#ifdef _ODI_OSSG_
//OS_MARK_SCHEMA_TYPE(ODBFunctionsDrawer);
//#endif

#endif
