/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

/*
** Generated by X-Designer
*/
#ifndef _UIScrolledText_h
#define _UIScrolledText_h

#define XD_MOTIF

#include <xdclass.h>

class UIScrolledText_c: public xd_XmForm_c {
public:
	virtual void create (Widget parent, char *widget_name = NULL);
protected:
	Widget UIScrolledText;
	Widget scrolledText_;
	Widget scrollbarH_;
	Widget scrollbarV_;
	Widget text_;
public:
	static void changedCB( Widget, XtPointer, XtPointer );
	virtual void changedCB( Widget, XtPointer ) = 0;
	static void activateCB( Widget, XtPointer, XtPointer );
	virtual void activateCB( Widget, XtPointer ) = 0;
};

typedef UIScrolledText_c *UIScrolledText_p;


extern UIScrolledText_p UIScrolledText;



#endif
