/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

/*
** Generated by X-Designer
*/
#ifndef _UIAsk_h
#define _UIAsk_h

#define XD_MOTIF

#include <xdclass.h>

class UIAsk_c: public xd_XmDialog_c {
public:
	virtual void create (Widget parent, char *widget_name = NULL);
protected:
	Widget UIAsk;
	Widget form_;
	Widget label_;
	Widget value_;
public:
	static void helpCB( Widget, XtPointer, XtPointer );
	virtual void helpCB( Widget, XtPointer ) = 0;
	static void cancelCB( Widget, XtPointer, XtPointer );
	virtual void cancelCB( Widget, XtPointer ) = 0;
	static void okCB( Widget, XtPointer, XtPointer );
	virtual void okCB( Widget, XtPointer ) = 0;
};

typedef UIAsk_c *UIAsk_p;


extern UIAsk_p UIAsk;


#endif
