/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

// File MethodCommand.h
// Baudouin Raoult - ECMWF Jul 99

#ifndef MethodCommand_H
#define MethodCommand_H

#include "Command.h"


class MethodCommand : public Command {
public:

	typedef void (IconObject::*Proc)();

// -- Exceptions
	// None

// -- Contructors

	MethodCommand(const string&,Proc);

// -- Destructor

	virtual ~MethodCommand(); // Change to virtual if base class

// -- Convertors
	// None

// -- Operators
	// None

// -- Methods

	virtual void execute(IconObject*);

// -- Overridden methods
	// None

// -- Class members
	// None

// -- Class methods

	static void execute(const string&,IconObject*);

protected:

// -- Members

	string name_;

// -- Methods
	
	// void print(ostream&) const; // Change to virtual if base class	

// -- Overridden methods
	// None

// -- Class members
	// None

// -- Class methods
	// None

private:

// No copy allowed

	MethodCommand(const MethodCommand&);
	MethodCommand& operator=(const MethodCommand&);

// -- Members

	Proc proc_;

// -- Methods
	// None

// -- Overridden methods
	// None

// -- Class members
	// None

// -- Class methods
	// None

// -- Friends

	//friend ostream& operator<<(ostream& s,const MethodCommand& p)
	//	{ p.print(s); return s; }

};

#endif
