/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

// File Dialog.h
// Baudouin Raoult - ECMWF Nov 98

#ifndef Dialog_H
#define Dialog_H

#ifndef Singleton_H
#include "Singleton.h"
#endif


template<class T,class U>
class Dialog : public U, public Singleton<T>  {
public:

	Dialog();

protected:

// -- Members

	bool ok_;
	bool stop_;

// -- Methods
	
	bool modal(const char*,bool);
	void show();

// -- Overridden methods
	// None

// -- Class members
	// None


private:

// No copy allowed

	Dialog(const Dialog<T,U>&);
	Dialog<T,U>& operator=(const Dialog<T,U>&);

// -- Members
	

// -- Methods


	virtual void helpCB( Widget, XtPointer ) ;
	virtual void cancelCB( Widget, XtPointer ) ;
	virtual void okCB( Widget, XtPointer ) ;

// -- Overridden methods
	// None

// -- Class methods
	// None

// -- Friends

	//friend ostream& operator<<(ostream& s,const Dialog& p)
	//	{ p.print(s); return s; }

};

#ifdef H_INCLUDES_CC
#include "Dialog.cc"
#endif


#endif
