/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

// File Netscape.h
// Baudouin Raoult - ECMWF Jun 00

#ifndef Netscape_H
#define Netscape_H


#include <string>
using std::string;

#include <Request.h>

class Path;

class Netscape {
public:

// -- Exceptions
	// None

// -- Contructors


// -- Destructor


// -- Convertors
	// None

// -- Operators
	// None

// -- Methods
	// None

// -- Overridden methods
	// None

// -- Class members

	static void showURL(const string&);
	static void showPage(const string&);
	static void showPage(const string&,const string&);
	static void savePage(const Path&,const string& = "HTML");

	static void load(request*);

// -- Class methods
	// None

	// Uncomment for persistent, remove otherwise
	// static os_typespec* get_os_typespec();

protected:

// -- Members


// -- Methods
	
	static void callNetscape(const Request&);

// -- Overridden methods
	// None

// -- Class members
	// None

// -- Class methods
	// None

private:

// No copy allowed

	Netscape(const Netscape&);
	Netscape& operator=(const Netscape&);

// -- Members


};

inline void destroy(Netscape**) {}

// If persistent, uncomment, otherwise remove
//#ifdef _ODI_OSSG_
//OS_MARK_SCHEMA_TYPE(Netscape);
//#endif

#endif
