/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

// File MacroRunTask.h
// Baudouin Raoult - ECMWF Jul 99

#ifndef MacroRunTask_H
#define MacroRunTask_H

#ifndef Task_H
#include "Task.h"
#endif

#ifndef Action_H
#include "Action.h"
#endif

#ifndef ReplyObserver_H
#include "ReplyObserver.h"
#endif

#include "TaskObserver.h"
#include "IconObject.h"
#include "Dependancy.h"


class MacroEditor;


class MacroRunTask : public Task, public ReplyObserver {
public:

// -- Exceptions
	// None

// -- Contructors

	MacroRunTask(MacroEditor&,const Action&,IconObject*,bool);

// -- Destructor

	virtual ~MacroRunTask(); // Change to virtual if base class

// -- Convertors
	// None

// -- Operators


// -- Methods
	virtual void addContext (const Request&) { };

// -- Overridden methods
	// None

// -- Class members
	// None

// -- Class methods
	// None

	// Uncomment for persistent, remove otherwise
	// static os_typespec* get_os_typespec();

protected:

// -- Members
	// None

// -- Methods
	
	 void print(ostream&) const; // Change to virtual if base class	

// -- Overridden methods
	// None

// -- Class members
	// None

// -- Class methods
	// None

private:

// No copy allowed

	MacroRunTask(const MacroRunTask&);
	MacroRunTask& operator=(const MacroRunTask&);

// -- Members

	MacroEditor& editor_;
	Action      action_;
	IconObjectH object_;
	bool        trace_;

// -- Methods


// -- Overridden methods

	// From Task
	virtual void start();
	// From ReplyObserver
	virtual void reply(const Request&, int);
	virtual void progress(const Request&);  
	virtual void message(const string&);
// -- Class members
	// None

// -- Class methods
	// None

// -- Friends

	//friend ostream& operator<<(ostream& s,const MacroRunTask& p)
	//	{ p.print(s); return s; }

};

inline void destroy(MacroRunTask**) {}

// If persistent, uncomment, otherwise remove
//#ifdef _ODI_OSSG_
//OS_MARK_SCHEMA_TYPE(MacroRunTask);
//#endif

#endif
