/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

#ifndef uPlotApp_H
#define uPlotApp_H

#include <MvRequest.h>
#include <MvProtocol.h>
#include <MvQApplication.h>

using namespace std;

class uPlotApp : public MvQApplication
{
public:
	// Constructor
	uPlotApp (int& ac, char** av, const char* name=NULL);

	// Destructor
	virtual ~uPlotApp();

	// Processing the request
	virtual void serve (MvProtocol&,MvRequest&);

private:
	// No copy allowed
	uPlotApp (const uPlotApp&);
	uPlotApp& operator=(const uPlotApp&){return *this;}
};

#endif
