/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

#ifndef FamilyDrawer_H
#include "FamilyDrawer.h"
#endif


#ifndef Editor_H
#include "Editor.h"
#endif

#ifndef XMLabel_H
#include "XMLabel.h"
#endif


#ifndef Tab_H
#include "Tab.h"
#endif

#ifndef IconObject_H
#include "IconObject.h"
#endif


#ifndef IconClass_H
#include "IconClass.h"
#endif
#ifndef FolderWindow_H
#include "FolderWindow.h"
#endif

#ifndef IconFactory_H
#include "IconFactory.h"
#endif
#ifndef EditorDrawerFactory_H
#include "EditorDrawerFactory.h"
#endif

#include "Drag.h"
#include "Folder.h"


FamilyDrawer::FamilyDrawer(IconClass& c, Widget p):
	RequestPanel(c)	 
{
     UIFamilyDrawer_c::create(p,(char*)c.name().c_str());
	 Manage();
}

FamilyDrawer::~FamilyDrawer()
{
}

void FamilyDrawer::show() 
{
 	Manage();
} 

void FamilyDrawer::update(Request& def) 
{
	request_ = def;
	call(RequestPanelItem::update);

}

void FamilyDrawer::hide() 
{
	Unmanage();
}
	
Widget FamilyDrawer::where() 
{
 	return rowcol_;
} 
