/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

// File ComputeEditor.h
// Baudouin Raoult - ECMWF Jul 99

#ifndef ComputeEditor_H
#define ComputeEditor_H


#ifndef BaseTextEditor_H
#include "BaseTextEditor.h"
#endif

class ComputeEditor : public BaseTextEditor {
public:

// -- Exceptions
	// None

// -- Contructors

	ComputeEditor(const IconClass&,const string&);

// -- Destructor

	virtual ~ComputeEditor(); // Change to virtual if base class

// -- Convertors
	// None

// -- Operators
	// None

// -- Methods
	// None

// -- Overridden methods
	// None

// -- Class members
	// None

// -- Class methods
	// None

	// Uncomment for persistent, remove otherwise
	// static os_typespec* get_os_typespec();

protected:

// -- Members
	// None

// -- Methods
	
	// void print(ostream&) const; // Change to virtual if base class	

// -- Overridden methods
	// None

// -- Class members
	// None

// -- Class methods
	// None

private:

// No copy allowed

	ComputeEditor(const ComputeEditor&);
	ComputeEditor& operator=(const ComputeEditor&);

// -- Members
	// None

// -- Methods
	// None

// -- Overridden methods

	virtual void apply();
	virtual void reset();
	virtual void close();
	virtual void merge(IconObject*);
	virtual void replace(IconObject*);

// -- Class members
	// None

// -- Class methods
	// None

// -- Friends

	//friend ostream& operator<<(ostream& s,const ComputeEditor& p)
	//	{ p.print(s); return s; }

};

inline void destroy(ComputeEditor**) {}

// If persistent, uncomment, otherwise remove
//#ifdef _ODI_OSSG_
//OS_MARK_SCHEMA_TYPE(ComputeEditor);
//#endif

#endif
