/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

/*
** Generated by X-Designer
*/
#ifndef _UIViewsDrawer_h
#define _UIViewsDrawer_h

#define XD_MOTIF

#include <xdclass.h>

class UIViewsDrawer_c: public xd_XmForm_c {
public:
	virtual void create (Widget parent, char *widget_name = NULL);
protected:
	Widget UIViewsDrawer;
	Widget drag_;
	Widget menu1_;
	Widget menu2_;
public:
	static void updateCB( Widget, XtPointer, XtPointer );
	virtual void updateCB( Widget, XtPointer ) = 0;
	static void openFolderCB( Widget, XtPointer, XtPointer );
	virtual void openFolderCB( Widget, XtPointer ) = 0;
	static void editCB( Widget, XtPointer, XtPointer );
	virtual void editCB( Widget, XtPointer ) = 0;
};

typedef UIViewsDrawer_c *UIViewsDrawer_p;


extern UIViewsDrawer_p UIViewsDrawer;


#endif
