/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

// File PaperDrawer.h
// Baudouin Raoult - ECMWF Jul 99

#ifndef PaperDrawer_H
#define PaperDrawer_H

#ifndef EditorDrawer_H
#include "EditorDrawer.h"
#endif

#include "UIPaperDrawer.h"

class ViewEditor;

class PaperDrawer : public XEditorDrawer<UIPaperDrawer_c> {
public:

// -- Exceptions
	// None

// -- Contructors

	PaperDrawer(ViewEditor&);

// -- Destructor

	virtual ~PaperDrawer(); // Change to virtual if base class

// -- Convertors
	// None

// -- Operators
	// None

// -- Methods
	// None

// -- Overridden methods
	// None

// -- Class members
	// None

// -- Class methods
	// None

	// Uncomment for persistent, remove otherwise
	// static os_typespec* get_os_typespec();

protected:

// -- Members
	// None

// -- Methods
	
	// void print(ostream&) const; // Change to virtual if base class	

// -- Overridden methods
  void setCB(Widget, XtPointer);
  void rotateCB(Widget, XtPointer);
  void a4CB(Widget, XtPointer);
  void a3CB(Widget, XtPointer);

// -- Class members
	// None

// -- Class methods
	// None

private:

// No copy allowed

	PaperDrawer(const PaperDrawer&);
	PaperDrawer& operator=(const PaperDrawer&);

// -- Members
  ViewEditor& editor_;

// -- Methods
  void setSize(double,double);

// -- Overridden methods
  virtual void edit(IconObject*);

// -- Class members
	// None

// -- Class methods
	// None

// -- Friends

	//friend ostream& operator<<(ostream& s,const PaperDrawer& p)
	//	{ p.print(s); return s; }

};

inline void destroy(PaperDrawer**) {}

// If persistent, uncomment, otherwise remove
//#ifdef _ODI_OSSG_
//OS_MARK_SCHEMA_TYPE(PaperDrawer);
//#endif

#endif
