/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

// File GridDrawer.h
// Baudouin Raoult - ECMWF Jul 99

#ifndef GridDrawer_H
#define GridDrawer_H

#ifndef EditorDrawer_H
#include "EditorDrawer.h"
#endif

#include "UIGridDrawer.h"

class ViewEditor;

class GridDrawer : public XEditorDrawer<UIGridDrawer_c> {
public:

// -- Exceptions
	// None

// -- Contructors

	GridDrawer(ViewEditor&);

// -- Destructor

	virtual ~GridDrawer(); // Change to virtual if base class

// -- Convertors
	// None

// -- Operators
	// None

// -- Methods
	// None

// -- Overridden methods
  void edit(IconObject*);
// -- Class members
	// None

// -- Class methods
	// None

	// Uncomment for persistent, remove otherwise
	// static os_typespec* get_os_typespec();

protected:

// -- Members
	// None

// -- Methods
	
	// void print(ostream&) const; // Change to virtual if base class	

// -- Overridden methods
	// None

// -- Class members
	// None

// -- Class methods
	// None

private:

// No copy allowed

	GridDrawer(const GridDrawer&);
	GridDrawer& operator=(const GridDrawer&);

// -- Members
  	ViewEditor& editor_;

	bool cm_x_;
	bool cm_y_;

// -- Methods

	void setGrid();

// -- Overridden methods

	  void setCB(Widget, XtPointer);
	  void showCB(Widget, XtPointer);
	  void snapCB(Widget, XtPointer);
	  void xEntryCB(Widget, XtPointer);
	  void yEntryCB(Widget, XtPointer);

// -- Class members
	// None

// -- Class methods
	// None

// -- Friends

	//friend ostream& operator<<(ostream& s,const GridDrawer& p)
	//	{ p.print(s); return s; }

};

inline void destroy(GridDrawer**) {}

// If persistent, uncomment, otherwise remove
//#ifdef _ODI_OSSG_
//OS_MARK_SCHEMA_TYPE(GridDrawer);
//#endif

#endif
