/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

#ifndef ARRAY_H
#define ARRAY_H

#define XtNround   "round"
#define XtCRound   "Round"
#define XtNrows    "rows"
#define XtNcolumns "columns"
#define XtCRowCol  "RowCol"

extern WidgetClass  arrayWidgetClass;

typedef struct _ArrayClassRec *ArrayWidgetClass;
typedef struct _ArrayRec      *ArrayWidget;

#ifdef _NO_PROTO

extern Widget CreateArray();

#else
 
#if defined(__cplusplus) || defined(c_plusplus)
extern "C" {
#endif

Widget    CreateArray(Widget,String,Arg*,int);
#if defined(__cplusplus) || defined(c_plusplus)
}
#endif
 
#endif /* _NO_PROTO */


#endif /* ARRAY_H */
