/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

/*
** Generated by X-Designer
*/
#ifndef _UICatalogDrawer_h
#define _UICatalogDrawer_h

#define XD_MOTIF

#include <xdclass.h>

class UICatalogDrawer_c: public xd_XmForm_c {
public:
	virtual void create (Widget parent, char *widget_name = NULL);
protected:
	Widget UICatalogDrawer;
	Widget message_;
public:
	static void helpCB( Widget, XtPointer, XtPointer );
	virtual void helpCB( Widget, XtPointer ) = 0;
	static void homeCB( Widget, XtPointer, XtPointer );
	virtual void homeCB( Widget, XtPointer ) = 0;
	static void grabCatalogCB( Widget, XtPointer, XtPointer );
	virtual void grabCatalogCB( Widget, XtPointer ) = 0;
	static void inCatalogCB( Widget, XtPointer, XtPointer );
	virtual void inCatalogCB( Widget, XtPointer ) = 0;
	static void topCatalogCB( Widget, XtPointer, XtPointer );
	virtual void topCatalogCB( Widget, XtPointer ) = 0;
};

typedef UICatalogDrawer_c *UICatalogDrawer_p;


extern UICatalogDrawer_p UICatalogDrawer;


#endif
