/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

// File DropSite.h
// Baudouin Raoult - ECMWF Jul 99

#ifndef DropSite_H
#define DropSite_H

#include <Xm/Xm.h>
#include <string>
using std::string;


class IconObject;
class IconClass;
class Dropping;
class MotifDrop;

class DropSite {
public:

// -- Exceptions
	// None

// -- Contructors

	DropSite();

// -- Destructor

	virtual ~DropSite(); // Change to virtual if base class

// -- Convertors
	// None

// -- Operators
	// None

// -- Methods

	void install(Widget);

	// ---

	virtual void drop(Dropping&) = 0;

// -- Overridden methods
	// None

// -- Class members
	// None

// -- Class methods


protected:


// -- Members
	// None


// -- Methods
	
	// void print(ostream&) const; // Change to virtual if base class	

// -- Overridden methods
	// None

// -- Class members
	// None

// -- Class methods
	// None

private:

// No copy allowed

	DropSite(const DropSite&);
	DropSite& operator=(const DropSite&);

// -- Members


// -- Methods


// -- Overridden methods


// -- Class members



// -- Class methods
	// None

	static void dropCB(Widget,XtPointer,XtPointer);

// -- Friends

	friend class MotifDrop;


};

#endif
