/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

/*
** Generated by X-Designer
*/
#ifndef _UIIconLine_h
#define _UIIconLine_h

#define XD_MOTIF

#include "xdclass.h"

class UIIconLine_c: public xd_XmForm_c {
public:
	virtual void create (Widget parent, char *widget_name = NULL);
protected:
	Widget UIIconLine;
	Widget drag_;
	Widget menu1_;
	Widget menu2_;
	Widget hidden_;
	Widget class_;
public:
	static void newIconCB( Widget, XtPointer, XtPointer );
	virtual void newIconCB( Widget, XtPointer ) = 0;
	static void openHiddenCB( Widget, XtPointer, XtPointer );
	virtual void openHiddenCB( Widget, XtPointer ) = 0;
	static void removeCB( Widget, XtPointer, XtPointer );
	virtual void removeCB( Widget, XtPointer ) = 0;
	static void editCB( Widget, XtPointer, XtPointer );
	virtual void editCB( Widget, XtPointer ) = 0;
};

typedef UIIconLine_c *UIIconLine_p;


extern UIIconLine_p UIIconLine;



#endif
