/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

/*
** Generated by X-Designer
*/
#ifndef _UIParamEditor_h
#define _UIParamEditor_h

#define XD_MOTIF

#include <xdclass.h>

class UIParamEditor_c: public xd_XmForm_c {
public:
	virtual void create (Widget parent, char *widget_name = NULL);
protected:
	Widget UIParamEditor;
	Widget drag_;
	Widget menu_;
	Widget Macro;
	Widget editor_;
public:
	static void removeCB( Widget, XtPointer, XtPointer );
	virtual void removeCB( Widget, XtPointer ) = 0;
	static void editCB( Widget, XtPointer, XtPointer );
	virtual void editCB( Widget, XtPointer ) = 0;
};

typedef UIParamEditor_c *UIParamEditor_p;


extern UIParamEditor_p UIParamEditor;


#endif
