/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

// File Help.h
// Baudouin Raoult - ECMWF Jun 00

#ifndef Help_H
#define Help_H


#include <string>
using std::string;

#include "Input.h"

class Help : public Input {
public:

// -- Exceptions
	// None

// -- Contructors


// -- Destructor


// -- Convertors
	// None

// -- Operators
	// None

// -- Methods
	// None

// -- Overridden methods
	// None

// -- Class members

	static void show(const string&);

// -- Class methods
	// None

	// Uncomment for persistent, remove otherwise
	// static os_typespec* get_os_typespec();

protected:

// -- Members


// -- Methods
	
	// void print(ostream&) const; // Change to virtual if base class	

// -- Overridden methods
	// None

// -- Class members
	// None

// -- Class methods
	// None

private:

// No copy allowed

	Help(const Help&);
	Help& operator=(const Help&);

// -- Members

// -- Methods
	
	virtual void start();

// -- Overridden methods

	// From Input

	virtual void ready(const char*);
	virtual void done(FILE*);

// -- Class members
	// None

// -- Class methods
	// None

// -- Friends

	//friend ostream& operator<<(ostream& s,const Help& p)
	//	{ p.print(s); return s; }

};

inline void destroy(Help**) {}

// If persistent, uncomment, otherwise remove
//#ifdef _ODI_OSSG_
//OS_MARK_SCHEMA_TYPE(Help);
//#endif

#endif
