/***************************** LICENSE START ***********************************

 Copyright 2015 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

#ifndef MvQStringLine_H
#define MvQStringLine_H

#include "MvQRequestPanelLine.h"

#include <QString>

class  QToolButton;

class  MvQLineEdit;
class  RequestPanel;
class  Parameter;

class MvQStringLine : public MvQRequestPanelLine
{
Q_OBJECT 

public:
	MvQStringLine(RequestPanel& owner,const Parameter& param);
	~MvQStringLine() {};	
	
	QString currentValue() {return QString();}
	void addValue(QString) {}
	
	void refresh(const vector<string>&);

public slots:
    void slotCleared();
    void slotTextEdited(const QString&);
	void slotHelperEdited(const vector<string>&);

protected:
	void dispatchChange();
	void updateHelper();	
	
	MvQLineEdit *lineEdit_;
	
};	
#endif
