/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

#ifndef MvQLayerContentsIcon_H
#define MvQLayerContentsIcon_H

#include <QPixmap>

#include "MvRequest.h"

class MgQIcon;

class MvQLayerContentsIcon
{
public:
	MvQLayerContentsIcon(const MgQIcon&);

	QString name() {return name_;}
	QPixmap pixmap() {return pix_;}
	QString path() {return path_;}
	QString id() {return id_;}
	void startEditor();
	void edited(MvRequest&);	
	bool canBeDeleted() {return canBeDeleted_;}
	bool canBeEdited() {return canBeEdited_;}
	bool canBeSaved() {return canBeSaved_;}
	bool isData() {return isData_;}
	
protected:		
  	QString name_;
	QString path_;
	QString type_;
	QString id_;
	QPixmap pix_;
	
	bool canBeDeleted_;
	bool canBeEdited_;
	bool canBeSaved_;
	bool isData_;
};

#endif
