/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

// File Command.h
// Baudouin Raoult - ECMWF Jul 99

#ifndef Command_H
#define Command_H

#include <string>
using std::string;

class IconObject;


class Command {
public:

// -- Exceptions
	// None

// -- Contructors

	Command(const string& name);

// -- Destructor

	virtual ~Command(); // Change to virtual if base class

// -- Convertors
	// None

// -- Operators
	// None

// -- Methods

	virtual void execute(IconObject*) = 0;

// -- Overridden methods
	// None

// -- Class members
	// None

// -- Class methods

	static void execute(const string&,IconObject*);

protected:

// -- Members

	string name_;

// -- Methods
	
	// void print(ostream&) const; // Change to virtual if base class	

// -- Overridden methods
	// None

// -- Class members
	// None

// -- Class methods
	// None

private:

// No copy allowed

	Command(const Command&);
	Command& operator=(const Command&);

// -- Members
	// None

// -- Methods
	// None

// -- Overridden methods
	// None

// -- Class members
	// None

// -- Class methods
	// None

// -- Friends

	//friend ostream& operator<<(ostream& s,const Command& p)
	//	{ p.print(s); return s; }

};

#endif
