/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

/*
** Generated by X-Designer
*/
#ifndef _stbase_h
#define _stbase_h

#define XD_MOTIF



extern Widget top;
extern Widget form;
extern Widget label_info;
extern Widget sub_form;
extern Widget search_option;
extern Widget search_menu;
extern Widget row_pos;
extern Widget label_lat;
extern Widget text_lat;
extern Widget label2;
extern Widget text_lon;
extern Widget label_thr;
extern Widget text_thr;
extern Widget row_area;
extern Widget label_n;
extern Widget text_n;
extern Widget label_w;
extern Widget text_w;
extern Widget label_s;
extern Widget text_s;
extern Widget label_e;
extern Widget text_e;
extern Widget row_name;
extern Widget text_name;
extern Widget row_ident;
extern Widget text_ident;
extern Widget row_wmo;
extern Widget text_wmo;
extern Widget eps_only;
extern Widget drag;
extern Widget list_stbase;
extern Widget plot_sel_button;
extern Widget plot_all_button;
extern Widget helpw;

extern void create_top (Display *display, char *app_name, int app_argc, char **app_argv)
;

#endif
