/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

// File MetviewUI.h
// Baudouin Raoult - ECMWF Jul 99

#ifndef MetviewUI_H
#define MetviewUI_H

#include <Xm/Xm.h>
#include <Metview.h>
#include <MvXApplication.h>

class MetviewUI : public MvXApplication {
public:

// -- Exceptions
	// None

// -- Contructors

	MetviewUI(XtAppContext, int argc, char** argv);

// -- Destructor

	~MetviewUI(); // Change to virtual if base class

// -- Convertors
	// None

// -- Operators
	// None

// -- Methods
	// None

// -- Overridden methods
	// None

// -- Class members
	// None

// -- Class methods

	static Widget root();
	static void load(request*);

protected:

// -- Members
	// None

// -- Methods
	
	// void print(ostream&) const; // Change to virtual if base class	

// -- Overridden methods
	// None

// -- Class members
	// None

// -- Class methods
	// None

private:

// No copy allowed

	MetviewUI(const MetviewUI&);
	MetviewUI& operator=(const MetviewUI&);

// -- Members
	// None

// -- Methods

	void loadResources();

// -- Overridden methods
	// None

// -- Class members

// -- Class methods
	// None

// -- Friends

	//friend ostream& operator<<(ostream& s,const MetviewUI& p)
	//	{ p.print(s); return s; }

};

inline void destroy(MetviewUI**) {}

// If persistent, uncomment, otherwise remove
//#ifdef _ODI_OSSG_
//OS_MARK_SCHEMA_TYPE(MetviewUI);
//#endif

#endif
