/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

// File XPixmap.h
// Baudouin Raoult - ECMWF Dec 98

#ifndef XPixmap_H
#define XPixmap_H


#include <Xm/Xm.h>
#include <X11/xpm.h>

#include <string>
using std::string;


#ifndef Path_H
#include "Path.h"
#endif


class XPixmap {
public:

// -- Exceptions
	// None

// -- Contructors

	XPixmap(const string&);
	XPixmap(const Path&);

// -- Destructor

	~XPixmap(); // Change to virtual if base class

// -- Convertors
	// None

// -- Operators
	// None
	operator Pixmap() { return pixels();  } 

// -- Methods

	Pixmap pixels();
	void setLabel(Widget);
	const string& name();

// -- Overridden methods
	// None

// -- Class members
	// None

// -- Class methods


	// Uncomment for persistent, remove otherwise
	// static os_typespec* get_os_typespec();

protected:

// -- Members
	// None

// -- Methods
	
	// void print(ostream&) const; // Change to virtual if base class	

// -- Overridden methods
	// None

// -- Class members
	// None

// -- Class methods
	// None

private:

// No copy allowed

	XPixmap(const XPixmap&);
	XPixmap& operator=(const XPixmap&);

// -- Members

	Pixmap pixmap_;
	Path   name_;

// -- Methods


// -- Overridden methods
	// None

// -- Class members

// -- Class methods
	// None

// -- Friends

	//friend ostream& operator<<(ostream& s,const XPixmap& p)
	//	{ p.print(s); return s; }

};

inline void destroy(XPixmap**) {}

// If persistent, uncomment, otherwise remove
//#ifdef _ODI_OSSG_
//OS_MARK_SCHEMA_TYPE(XPixmap);
//#endif

#endif
