/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

// File FamilyDrawer.h
// Baudouin Raoult - ECMWF Jul 99

#ifndef FamilyDrawer_H
#define FamilyDrawer_H

#ifndef EditorDrawer_H
#include "EditorDrawer.h"
#endif

#ifndef IconHolder_H
#include "IconHolder.h"
#endif

#ifndef UIFamilyDrawer_H  
#include "UIFamilyDrawer.h"
#endif

#ifndef Folder_H  
#include "Folder.h"
#endif
#ifndef FamilyEditor_H  
#include "FamilyEditor.h"
#endif


#ifndef RequestPanel_H  
#include "RequestPanel.h"
#endif


class FamilyDrawer : public UIFamilyDrawer_c, 	public RequestPanel {
public:

// -- Exceptions
	// None

// -- Contructors

	FamilyDrawer(const IconClass&, Widget p);

// -- Destructor

	virtual ~FamilyDrawer(); // Change to virtual if base class

// -- Convertors
	// None

// -- Operators
	// None

// -- Methods
	// None
	 virtual Widget where();
	 void show();
	 void hide();
	 void update(Request& r);

// -- Overridden methods
	// None

// -- Class members
	// None 


// -- Class methods
	// None 


	// Uncomment for persistent, remove otherwise
	// static os_typespec* get_os_typespec();

protected:

// -- Members
	// None

// -- Methods

	
	// void print(ostream&) const; // Change to virtual if base class	

// -- Overridden methods
	// None

// -- Class members
	// None

// -- Class methods
	// None

private:

// No copy allowed

	FamilyDrawer(const FamilyDrawer&);
	FamilyDrawer& operator=(const FamilyDrawer&);

// -- Members
	// None	

// -- Methods

// -- Overridden methods
  

// -- Class members
	// None

// -- Class methods
	// None

// -- Friends

	//friend ostream& operator<<(ostream& s,const FamilyDrawer& p)
	//	{ p.print(s); return s; }

};

inline void destroy(FamilyDrawer**) {}

// If persistent, uncomment, otherwise remove
//#ifdef _ODI_OSSG_
//OS_MARK_SCHEMA_TYPE(FamilyDrawer);
//#endif

#endif
