/***************************** LICENSE START ***********************************

 Copyright 2014 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

#ifndef MvQColourLine_H
#define MvQColourLine_H

#include "MvQRequestPanelLine.h"
#include "Parameter.h"

#include <QPixmap>
#include <QString>

class QColor;
class QComboBox;

class  RequestPanel;

class MvQColourLine : public MvQRequestPanelLine, public ParameterScanner
{
Q_OBJECT  
  
public:
	MvQColourLine(RequestPanel& owner,const Parameter& param);
	~MvQColourLine() {};	
	
	QString currentValue() {return QString();}
	void addValue(QString) {}
	
	void refresh(const vector<string>&);

public slots:	
        void slotCurrentChanged(int);	
	void slotHelperEdited(const vector<string>&); 
	void slotHelperEditConfirmed();
	
protected:
	void next(const Parameter&, const char* first,const char* second); 
	void addColour(QColor,QString,QString,bool setCurrent=false);
	void setCustomColour(QColor,QString,QString,bool setCurrent=false);
	void updateHelper();
	void setColour(int,QColor);
	 
	QComboBox *colCb_;
	QPixmap pix_;
};	

#endif
