/***************************** LICENSE START ***********************************

 Copyright 2014 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

#ifndef Dependancy_H
#define Dependancy_H

#include "Counted.h"

class IconObject;
class Task;
class Action;
class Request;

class Dependancy : public virtual Counted 
{
public:
	Dependancy();
	virtual ~Dependancy(); 

	virtual Task* action(const Action&)   = 0;
	virtual void  success(const Request&) = 0;
	virtual void  failure() =0;

};

class DependancyH : public Handle<Dependancy> {
public:
	DependancyH(Dependancy* o = 0) : Handle<Dependancy>(o) {}
};

#endif
