/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/


/*
** Generated by X-Designer 
*/

extern Widget text_top;
extern Widget text_form;
extern Widget text;

extern Widget folder_top;
extern Widget folder_form;
extern Widget folder_drag;

extern Widget top;
extern Widget form;
extern Widget drag;

extern void create_text_top (Widget);
extern void create_folder_top (Widget);
extern void create_top (Display *, char *, int, char **);
