/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

// File RootFolder.h
// Baudouin Raoult - ECMWF Jul 99

#ifndef RootFolder_H
#define RootFolder_H

#include "Folder.h"

class RootFolder : public Folder {
public:

// -- Exceptions
	// None

// -- Contructors

	RootFolder();

// -- Destructor

	virtual ~RootFolder(); // Change to virtual if base class

// -- Convertors
	// None

// -- Operators
	// None

// -- Methods
	// None

// -- Overridden methods
	// None

// -- Class members
	// None

// -- Class methods
	// None

	// Uncomment for persistent, remove otherwise
	// static os_typespec* get_os_typespec();

protected:

// -- Members
	// None

// -- Methods
	
	// void print(ostream&) const; // Change to virtual if base class	

// -- Overridden methods
	// None

// -- Class members
	// None

// -- Class methods
	// None

private:

// No copy allowed

	RootFolder(const RootFolder&);
	RootFolder& operator=(const RootFolder&);

// -- Members
	// None

// -- Methods
	// None

// -- Overridden methods

	virtual Path path() const;
	virtual Path dotPath() const;
	virtual string fullName() const;
	virtual Folder* parent() const;
	virtual Path logPath();

// -- Class members
	// None

// -- Class methods
	// None

// -- Friends

	//friend ostream& operator<<(ostream& s,const RootFolder& p)
	//	{ p.print(s); return s; }

};

inline void destroy(RootFolder**) {}

// If persistent, uncomment, otherwise remove
//#ifdef _ODI_OSSG_
//OS_MARK_SCHEMA_TYPE(RootFolder);
//#endif

#endif
