/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

// File Message.h
// Geir Austad - ECMWF May 01

#ifndef Message_H
#define Message_H

// Forward declarations

// class ostream;
// typedef class _Pvts os_typespec; // Remove if not persistant

// 
#ifndef Metview_H
#include <Metview.h>
#define Metview_H
#endif

class Message : public MvMessage
{
public:
  Message(const char * name) : MvMessage(name) {}
  virtual ~Message() {}
	
private:

  // No copy allowed
  Message(const Message&);
  Message& operator=(const Message&);
};



inline void destroy(Message**) {}

#endif
