# © Copyright 1996-2012 ECMWF.
# 
# This software is licensed under the terms of the Apache Licence Version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0. 
# In applying this licence, ECMWF does not waive the privileges and immunities 
# granted to it by virtue of its status as an intergovernmental organisation nor
# does it submit to any jurisdiction.

#
# PARAM {
#    value1 ;  alias1 ; alias; alias ....
#    /  if list ok
#    @ any
#    " quotted string
#    * number 
#    BY
#    TO
# } = default
#
#

GRIB;  Local grib definition; ITALY
{

	COUNTRY {
		IT
	} 

	MODEL {
	   LM
	   HRM
	   WAM
	} 

	DOMAIN {
	   ITALY
	   EUROATL
	   EUROPE
	   MEDITERRANEAN
	}

	BCMODEL {
		GME
		IFS
	}

	ICMODEL {
		GME
		IFS
		MVOI
		NUDGING
		3DVAR
	}

    LEVTYPE { 
		SFC
		ML
		PL
		* 
		/	
		ALL
	} 

} ! END OF GRIB



ARCHIVE ;   Archives data   ;   ITALY
{

    CLASS  { 
		MS
	} = MS

	COUNTRY {
		IT
	} = IT

	MODEL {
	   LM
	   HRM
	   WAM
	} = LM

	BCMODEL {
		GME
		IFS
	} = IFS

	ICMODEL {
		GME
		IFS
		MVOI
		NUDGING
		3DVAR
	} = IFS

	DOMAIN {
	   ITALY
	   EUROATL
	   EUROPE
	   MEDITERRANEAN
	}

	STREAM {
		 &_DUMMY&STREAM
	} = DA

	TYPE {
	 	&_DUMMY&TYPE
	} = AN

    EXPVER  
    {
        *       
    } 
    
    LEVTYPE { 
		SFC
		ML
		PL
		* 
		/	
		ALL
	} 

    PARAM { 
		* 
		/
		ALL
	} 

	TIME {
		*
		/
		ALL
	}

	DATE {
		*
		/
		ALL
	}

    STEP { 
		* 
		/
		ALL
	} = 00


    LEVELIST  
    {
		TO ; TO
		BY ; BY 
        OFF ;   OFF
        *       
        /       
		ALL
    } 
    

    SOURCE
    {
		@
		/
        "   
    } = ""
    
    DISP
    {
        NEW ;   N
        OLD ;   OLD
		FAIL 
		REMOVE
    } = NEW
    
	DATABASE
	{
		@
		/
	} 

	EXPECT [ priority = 0 ]
	{
		ANY
		*
		OFF ; OFF
	} = OFF


	PASSWORD {
		@
	}

	REPRES {
		@
	}

	GRIB {
	@
	}

	DUPLICATE {
	@
	}

} ! END OF ARCHIVE

LIST ;   Lists data   ;   ITALY
{

    CLASS  { 
		MS
	} = MS

	COUNTRY {
		IT
	} = IT

	MODEL {
	   LM
	   HRM
	   WAM
	   ALL
	} = LM

	BCMODEL {
		GME
		IFS
	   ALL
	} = IFS

	ICMODEL {
		GME
		IFS
		MVOI
		NUDGING
		3DVAR
	   ALL
	} = IFS

	DOMAIN {
	   ITALY
	   EUROATL
	   EUROPE
	   MEDITERRANEAN
	   ALL
	}

	STREAM {
		 &_DUMMY&STREAM
	   ALL
	} = DA

	TYPE {
	 	&_DUMMY&TYPE
	   ALL
	} = AN

    EXPVER  
    {
		@
        *       
    } 
    
    LEVTYPE { 
		SFC
		ML
		PL
		* 
		/	
		ALL
	} 

    PARAM { 
		* 
		/
		ALL
	} 

	TIME {
		*
		/
		ALL
	}

	DATE {
		*
		/
		ALL
	}

    STEP { 
		* 
		/
		ALL
	} 

    LEVELIST  
    {
		TO ; TO
		BY ; BY 
        OFF ;   OFF
        *       
        /       
		ALL
    } 
    

    TARGET
    {
		@
		/
        "   
    } = ""
    
    DISP
    {
        NEW ;   N
        OLD ;   OLD
		FAIL 
		REMOVE
    } = NEW
    
	DATABASE
	{
		@
		/
	} 

	EXPECT [ priority = 0 ]
	{
		ANY
		*
		OFF ; OFF
	} = OFF


	PASSWORD {
		@
	}

	REPRES {
		@
	}

	OUTPUT {
		TABLE
		BROWSE
		COST
		SEARCH
		TREE
	} = TABLE

	HIDE   { @ / }

	SHOW   { @ / }

} ! END OF LIST

RETRIEVE ;   Retrieves data   ;   ITALY
{

    CLASS  { 
		MS
	} = MS

	COUNTRY {
		IT
	} = IT

	MODEL {
	   LM
	   HRM
	   WAM
	} = LM

	BCMODEL {
		GME
		IFS
	} = IFS

	ICMODEL {
		GME
		IFS
		MVOI
		NUDGING
		3DVAR
	} = IFS

	DOMAIN {
	   ITALY
	   EUROATL
	   EUROPE
	   MEDITERRANEAN
	}

	STREAM {
		 &_DUMMY&STREAM
	} = DA

	TYPE {
	 	&_DUMMY&TYPE
	} = AN

    EXPVER  
    {
        *       
    }  = 1
    
    LEVTYPE { 
		SFC
		ML
		PL
		* 
		/
		ALL
	}  = PL

    PARAM { 
		* 
		/
		ALL
	}  = 11

	TIME {
		*
		/
		ALL
		TO
	} = 12

	DATE {
		*
		/
		ALL
		TO
	} = 0

    STEP { 
		TO
		BY
		* 
		/
		ALL
	} = 0

    LEVELIST  
    {
		TO ; TO
		BY ; BY 
        OFF ;   OFF
        *       
        /       
		ALL
    } = 0
    
    SOURCE
    {
		@
		/
        "   
    } = ""
    

    TARGET
    {
		@
		/
        "   
    } = ""

	FIELDSET
	{
		@
		OFF
	}

	GRIB 
	{
		ORIGINAL
		ECMWF
	} = ECMWF
    
    DISP
    {
        NEW ;   N
        OLD ;   OLD
		FAIL 
		REMOVE
    }
    
	DATABASE
	{
		@
		/
	} 

	EXPECT [ priority = 0 ]
	{
		ANY
		*
		OFF ; OFF
	} = OFF


	PASSWORD {
		@
	}

	REPRES {
		@
	}

	GRID {
		*
		/
		OFF
	} = OFF

	PADDING {
		*
		OFF
	}

	YEAR { @ }

} ! END OF RETRIEVE


FLUSH ;   Flushes data   ;   ITALY
{

    CLASS  { 
		MS
	} = MS

	COUNTRY {
		IT
	} = IT

	MODEL {
	   LM
	   HRM
	   WAM
	} = LM

	BCMODEL {
		GME
		IFS
	} = IFS

	ICMODEL {
		GME
		IFS
		MVOI
		NUDGING
		3DVAR
	} = IFS

	DOMAIN {
	   ITALY
	   EUROATL
	   EUROPE
	   MEDITERRANEAN
	}

	STREAM {
		 &_DUMMY&STREAM
	} = DA

	TYPE {
	 	&_DUMMY&TYPE
	} = AN

    EXPVER  
    {
        *       
    } 
    
    LEVTYPE { 
		SFC
		ML
		PL
		* 
		/	
		ALL
	} 

    PARAM { 
		* 
		/
		ALL
	} 

	TIME {
		*
		/
		ALL
	}

	DATE {
		*
		/
		ALL
	}

    STEP { 
		* 
		/
		ALL
	} 


    LEVELIST  
    {
		TO ; TO
		BY ; BY 
        OFF ;   OFF
        *       
        /       
		ALL
    } 
    

    SOURCE
    {
		@
		/
        "   
    } = ""
    
    DISP
    {
        NEW ;   N
        OLD ;   OLD
		FAIL 
		REMOVE
    } = NEW
    
	DATABASE
	{
		@
		/
	} 

	EXPECT [ priority = 0 ]
	{
		ANY
		*
		OFF ; OFF
	} = OFF


	PASSWORD {
		@
	}

	REPRES {
		@
	}

} ! END OF FLUSH
