/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

// File Runnable.h
// Baudouin Raoult - ECMWF Oct 98

#ifndef Runnable_H
#define Runnable_H

#include <Xm/Xm.h>

#ifdef NO_BOOL
#include "bool.h"
#endif

// Headers
// #ifndef   machine_H
// #include <machine.h>
// #endif

// Forward declarations

// class ostream;
// typedef class _Pvts os_typespec; // Remove if not persistant



// 

class Runnable {
public:

// -- Exceptions
	// None

// -- Contructors

	Runnable();

// -- Destructor

	virtual ~Runnable(); // Change to virtual if base class

// -- Convertors
	// None

// -- Operators
	// None

// -- Methods
	// None

	void enable();
	void disable();
	bool actived() { return actived_; }

	virtual void run() = 0;

// -- Overridden methods
	// None

// -- Class members
	// None

// -- Class methods
	// None

	// Uncomment for persistent, remove otherwise
	// static os_typespec* get_os_typespec();

protected:

// -- Members
	// None

// -- Methods
	
	// void print(ostream&) const; // Change to virtual if base class	

// -- Overridden methods
	// None

// -- Class members
	// None

// -- Class methods
	// None

private:

// No copy allowed

	Runnable(const Runnable&);
	Runnable& operator=(const Runnable&);

// -- Members

	Boolean actived_;

// -- Methods
	// None

// -- Overridden methods
	// None

// -- Class members
	// None

// -- Class methods
	// None
	static Boolean workCB(XtPointer);

// -- Friends

	//friend ostream& operator<<(ostream& s,const Runnable& p)
	//	{ p.print(s); return s; }

};


#endif
