/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

// File IconDrawer.h
// Baudouin Raoult - ECMWF Jul 99

#ifndef IconDrawer_H
#define IconDrawer_H

#include "UIIconDrawer.h"

#ifndef FolderViewer_H
#include "FolderViewer.h"
#endif

class FolderDrawers;

class IconDrawer : public UIIconDrawer_c, public FolderViewer {
public:

// -- Exceptions
	// None

// -- Contructors

	IconDrawer(FolderDrawers&,Folder*);

// -- Destructor

	virtual ~IconDrawer(); // Change to virtual if base class

// -- Convertors
	// None

// -- Operators
	// None

// -- Methods
	// None

// -- Overridden methods
	// None

// -- Class members
	// None

// -- Class methods
	// None

	// Uncomment for persistent, remove otherwise
	// static os_typespec* get_os_typespec();

protected:

// -- Members
	// None

// -- Methods
	
	// void print(ostream&) const; // Change to virtual if base class	

// -- Overridden methods
	// None

// -- Class members
	// None

// -- Class methods
	// None

private:

// No copy allowed

	IconDrawer(const IconDrawer&);
	IconDrawer& operator=(const IconDrawer&);

// -- Members
	// None

	FolderDrawers& owner_;

// -- Methods
	// None

// -- Overridden methods

	// From DragWindow

	virtual Widget menu();
	virtual bool dropCopy() { return true; }

	// From XD

     virtual void deleteDrawerCB(Widget, XtPointer);
     virtual void renameDrawerCB(Widget, XtPointer);
     virtual void updateDrawerCB(Widget, XtPointer);
     virtual void newDrawerCB(Widget, XtPointer);
     virtual void newIconCB(Widget, XtPointer);
     virtual void deleteCB(Widget, XtPointer);
     virtual void editCB(Widget, XtPointer);
     virtual void openFolderCB(Widget, XtPointer);

// -- Class members
	// None

// -- Class methods
	// None

// -- Friends

	//friend ostream& operator<<(ostream& s,const IconDrawer& p)
	//	{ p.print(s); return s; }

};

inline void destroy(IconDrawer**) {}

// If persistent, uncomment, otherwise remove
//#ifdef _ODI_OSSG_
//OS_MARK_SCHEMA_TYPE(IconDrawer);
//#endif

#endif
