/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

// File Resources.h
// Baudouin Raoult - ECMWF Oct 99

#ifndef Resources_H
#define Resources_H

class IconObject;

#include "inc_stl.h"
using std::string;


class Request;

class Resources {
public:

	static void get(const string&,list<string>&,IconObject* = 0);
	static void set(const string&,const list<string>&,IconObject* = 0);

	static void get(const string&,vector<string>&,IconObject* = 0);
	static void set(const string&,const vector<string>&,IconObject* = 0);

	static void get(const string&,string&,IconObject* = 0);
	static void set(const string&,const string&,IconObject* = 0);

	static void get(const string&,long&,IconObject* = 0);
	static void set(const string&,long,IconObject* = 0);

	static void set(const Request&,IconObject*);
	static void sync();

protected:

// -- Members
	// None

// -- Methods
	
	// void print(ostream&) const; // Change to virtual if base class	

// -- Overridden methods
	// None

// -- Class members
	// None

// -- Class methods
	// None

private:

// No copy allowed

	Resources(const Resources&);
	Resources& operator=(const Resources&);

// -- Members
	// None

// -- Methods
	// None

// -- Overridden methods
	// None

// -- Class members
	// None

// -- Class methods
	// None

// -- Friends

	//friend ostream& operator<<(ostream& s,const Resources& p)
	//	{ p.print(s); return s; }

};

inline void destroy(Resources**) {}

// If persistent, uncomment, otherwise remove
//#ifdef _ODI_OSSG_
//OS_MARK_SCHEMA_TYPE(Resources);
//#endif

#endif
