/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

/*
** Generated by X-Designer
*/
/*
**LIBS: -lXm -lXt -lX11
*/

#include <stdlib.h>
#include <X11/Xatom.h>
#include <X11/Intrinsic.h>
#include <X11/Shell.h>

#include <Xm/Xm.h>
#include <Xm/CascadeB.h>
#include <Xm/DialogS.h>
#include <Xm/DrawingA.h>
#include <Xm/Form.h>
#include <Xm/Label.h>
#include <Xm/PushB.h>
#include <Xm/RowColumn.h>
#include <Xm/ScrollBar.h>
#include <Xm/ScrolledW.h>
#include <Xm/Separator.h>
#include <Drag.h>
#include <Tab.h>

#include <Xm/Protocols.h>
#include "UIFolderWindow.h"

UIFolderWindow_p UIFolderWindow = (UIFolderWindow_p) NULL;



void UIFolderWindow_c::create (Display *display, char *app_name, int app_argc, char **app_argv, char *app_class_name)
{
	Widget children[13];      /* Children to manage */
	Arg al[64];                    /* Arg List */
	register int ac = 0;           /* Arg Count */
	XrmValue from_value, to_value; /* For resource conversion */
	XmString xmstrings[16];    /* temporary storage for XmStrings */
	Widget scrolledWin1 = (Widget)NULL;
	Widget scrollbar1 = (Widget)NULL;
	Widget scrollbar2 = (Widget)NULL;
	Widget label1 = (Widget)NULL;
	Widget separator2 = (Widget)NULL;
	Widget button10 = (Widget)NULL;
	Widget button11 = (Widget)NULL;
	Widget button13 = (Widget)NULL;
	Widget button14 = (Widget)NULL;
	Widget button101 = (Widget)NULL;
	Widget separator5 = (Widget)NULL;
	Widget button12 = (Widget)NULL;
	Widget button15 = (Widget)NULL;
	Widget button16 = (Widget)NULL;
	Widget button6 = (Widget)NULL;
	Widget separator4 = (Widget)NULL;
	Widget button5 = (Widget)NULL;
	Widget button8 = (Widget)NULL;
	Widget button9 = (Widget)NULL;
	Widget separator7 = (Widget)NULL;
	Widget button22 = (Widget)NULL;
	Widget button24 = (Widget)NULL;
	Widget separator10 = (Widget)NULL;
	Widget button20 = (Widget)NULL;
	Widget separator9 = (Widget)NULL;
	Widget button21 = (Widget)NULL;
	Widget button17 = (Widget)NULL;
	Widget menuBar1 = (Widget)NULL;
	Widget cascade1 = (Widget)NULL;
	Widget menu1 = (Widget)NULL;
	Widget button18 = (Widget)NULL;
	Widget separator8 = (Widget)NULL;
	Widget button23 = (Widget)NULL;
	Widget separator11 = (Widget)NULL;
	Widget button2 = (Widget)NULL;
	Widget cascade3 = (Widget)NULL;
	Widget menu2 = (Widget)NULL;
	Widget button1 = (Widget)NULL;
	Widget button3 = (Widget)NULL;
	Widget separator3 = (Widget)NULL;
	Widget button4 = (Widget)NULL;
	Widget button19 = (Widget)NULL;
	Widget separator6 = (Widget)NULL;
	Widget button7 = (Widget)NULL;
	Widget cascade5 = (Widget)NULL;
	Widget cascade4 = (Widget)NULL;
	Widget separator1 = (Widget)NULL;
	Widget menu3 = (Widget)NULL;
	Widget button25 = (Widget)NULL;
	Widget separator12 = (Widget)NULL;
	Widget button26 = (Widget)NULL;
	Widget button27 = (Widget)NULL;
	Widget button28 = (Widget)NULL;
	Widget button31 = (Widget)NULL;
	Widget separator13 = (Widget)NULL;
	Widget button29 = (Widget)NULL;
	Widget button30 = (Widget)NULL;

	if ( !app_class_name )
		app_class_name = "XApplication";

	XtSetArg(al[ac], XmNallowShellResize, FALSE); ac++;
	XtSetArg(al[ac], XmNdeleteResponse, XmDO_NOTHING); ac++;
	XtSetArg(al[ac], XmNargc, app_argc); ac++;
	XtSetArg(al[ac], XmNargv, app_argv); ac++;
	UIFolderWindow = XtAppCreateShell ( app_name, app_class_name, applicationShellWidgetClass, display, al, ac );
	ac = 0;
	_xd_rootwidget = UIFolderWindow;
	XmAddWMProtocolCallback( UIFolderWindow, XmInternAtom(XtDisplay(UIFolderWindow), "WM_DELETE_WINDOW", False), closeCB, (XtPointer) this);
	XtSetArg(al[ac], XmNwidth, 500); ac++;
	XtSetArg(al[ac], XmNheight, 500); ac++;
	XtSetArg(al[ac], XmNautoUnmanage, FALSE); ac++;
	form_ = XmCreateForm ( UIFolderWindow, "form_", al, ac );
	ac = 0;
	XtSetArg(al[ac], XmNshadowThickness, 0); ac++;
	XtSetArg(al[ac], XmNscrollBarDisplayPolicy, XmSTATIC); ac++;
	XtSetArg(al[ac], XmNscrollingPolicy, XmAUTOMATIC); ac++;
	scrolledWin1 = XmCreateScrolledWindow ( form_, "scrolledWin1", al, ac );
	ac = 0;

	XtSetArg(al[ac], XmNhorizontalScrollBar, &scrollbar1 ); ac++;
	XtSetArg(al[ac], XmNverticalScrollBar, &scrollbar2 ); ac++;
	XtGetValues(scrolledWin1, al, ac );
	ac = 0;
	XtSetArg(al[ac], XmNautoPlace, FALSE); ac++;
	XtSetArg(al[ac], XmNeditNames, TRUE); ac++;
	XtSetArg(al[ac], XmNwidth, 20000); ac++;
	XtSetArg(al[ac], XmNheight, 20000); ac++;
	XtSetArg(al[ac], XmNtraversalOn, TRUE); ac++;
	XtSetArg(al[ac], XmNnavigationType, XmTAB_GROUP); ac++;
	if (DefaultDepthOfScreen(DefaultScreenOfDisplay(display)) != 1) {
	from_value.addr = "red" ;
	from_value.size = strlen( from_value.addr ) + 1;
	to_value.addr = NULL;
	XtConvertAndStore (scrolledWin1, XmRString, &from_value, XmRPixel, &to_value);
	if ( to_value.addr )
	{
		XtSetArg(al[ac], XmNhighlightColor, *(unsigned int *)to_value.addr); ac++;
	}
	}
	drag_ = XtCreateWidget("drag_", dragWidgetClass, scrolledWin1, al, ac);
	ac = 0;
	commands_ = XmCreatePopupMenu ( drag_, "commands_", al, ac );
	label1 = XmCreateLabel ( commands_, "title", al, ac );
	separator2 = XmCreateSeparator ( commands_, "separator2", al, ac );
	button10 = XmCreatePushButton ( commands_, "execute", al, ac );
	button11 = XmCreatePushButton ( commands_, "visualise", al, ac );
	button13 = XmCreatePushButton ( commands_, "examine", al, ac );
	button14 = XmCreatePushButton ( commands_, "save", al, ac );
	button101 = XmCreatePushButton ( commands_, "analyse", al, ac );
	separator5 = XmCreateSeparator ( commands_, "separator5", al, ac );
	button12 = XmCreatePushButton ( commands_, "edit", al, ac );
	button15 = XmCreatePushButton ( commands_, "duplicate", al, ac );
	button16 = XmCreatePushButton ( commands_, "delete", al, ac );
	button6 = XmCreatePushButton ( commands_, "empty", al, ac );
	separator4 = XmCreateSeparator ( commands_, "separator4", al, ac );
	button5 = XmCreatePushButton ( commands_, "output", al, ac );
	menu_ = XmCreatePopupMenu ( drag_, "menu_", al, ac );
	button8 = XmCreatePushButton ( menu_, "New icon...", al, ac );
	button9 = XmCreatePushButton ( menu_, "New link...", al, ac );
	separator7 = XmCreateSeparator ( menu_, "separator7", al, ac );
	button22 = XmCreatePushButton ( menu_, "Sort by name", al, ac );
	button24 = XmCreatePushButton ( menu_, "Sort by class", al, ac );
	separator10 = XmCreateSeparator ( menu_, "separator10", al, ac );
	button20 = XmCreatePushButton ( menu_, "Rescan folder", al, ac );
	separator9 = XmCreateSeparator ( menu_, "separator9", al, ac );
	button21 = XmCreatePushButton ( menu_, "Toggle icon size", al, ac );
	button17 = XmCreatePushButton ( menu_, "Show last icon created", al, ac );
	XtSetArg(al[ac], XmNdrawer, TRUE); ac++;
	drawer_ = XtCreateWidget("drawer_", tabWidgetClass, form_, al, ac);
	ac = 0;
	menuBar1 = XmCreateMenuBar ( form_, "menuBar1", al, ac );
	cascade1 = XmCreateCascadeButton ( menuBar1, "File", al, ac );
	menu1 = XmCreatePulldownMenu ( menuBar1, "Exit metview", al, ac );
	button18 = XmCreatePushButton ( menu1, "Preferences...", al, ac );
	separator8 = XmCreateSeparator ( menu1, "separator8", al, ac );
	button23 = XmCreatePushButton ( menu1, "Messages...", al, ac );
	separator11 = XmCreateSeparator ( menu1, "separator11", al, ac );
	close_ = XmCreatePushButton ( menu1, "Close", al, ac );
	button2 = XmCreatePushButton ( menu1, "Exit...", al, ac );
	cascade3 = XmCreateCascadeButton ( menuBar1, "View", al, ac );
	menu2 = XmCreatePulldownMenu ( menuBar1, "menu2", al, ac );
	button1 = XmCreatePushButton ( menu2, "Sort by name", al, ac );
	button3 = XmCreatePushButton ( menu2, "Sort by class", al, ac );
	separator3 = XmCreateSeparator ( menu2, "separator3", al, ac );
	button4 = XmCreatePushButton ( menu2, "Toggle icon size", al, ac );
	button19 = XmCreatePushButton ( menu2, "Rescan folder", al, ac );
	separator6 = XmCreateSeparator ( menu2, "separator6", al, ac );
	button7 = XmCreatePushButton ( menu2, "Show last icon created", al, ac );
	cascade5 = XmCreateCascadeButton ( menuBar1, "Tools", al, ac );
	tools_ = XmCreatePulldownMenu ( menuBar1, "tools_", al, ac );
	cascade4 = XmCreateCascadeButton ( menuBar1, "Folders", al, ac );
	recent_ = XmCreatePulldownMenu ( menuBar1, "recent_", al, ac );
	xmstrings[0] = XmStringCreateLtoR ( "Main Folder", (XmStringCharSet)XmFONTLIST_DEFAULT_TAG );
	XtSetArg(al[ac], XmNlabelString, xmstrings[0]); ac++;
	top_ = XmCreatePushButton ( recent_, "/", al, ac );
	ac = 0;
	XmStringFree ( xmstrings [ 0 ] );
	xmstrings[0] = XmStringCreateLtoR ( "Parent Folder", (XmStringCharSet)XmFONTLIST_DEFAULT_TAG );
	XtSetArg(al[ac], XmNlabelString, xmstrings[0]); ac++;
	up_ = XmCreatePushButton ( recent_, "..", al, ac );
	ac = 0;
	XmStringFree ( xmstrings [ 0 ] );
	xmstrings[0] = XmStringCreateLtoR ( "Defaults", (XmStringCharSet)XmFONTLIST_DEFAULT_TAG );
	XtSetArg(al[ac], XmNlabelString, xmstrings[0]); ac++;
	defaults_ = XmCreatePushButton ( recent_, "/System/Defaults", al, ac );
	ac = 0;
	XmStringFree ( xmstrings [ 0 ] );
	separator1 = XmCreateSeparator ( recent_, "separator1", al, ac );
	help_ = XmCreateCascadeButton ( menuBar1, "Help", al, ac );
	menu3 = XmCreatePulldownMenu ( menuBar1, "menu3", al, ac );
	xmstrings[0] = XmStringCreateLtoR ( "Index...", (XmStringCharSet)XmFONTLIST_DEFAULT_TAG );
	XtSetArg(al[ac], XmNlabelString, xmstrings[0]); ac++;
	button25 = XmCreatePushButton ( menu3, "index", al, ac );
	ac = 0;
	XmStringFree ( xmstrings [ 0 ] );
	separator12 = XmCreateSeparator ( menu3, "separator12", al, ac );
	xmstrings[0] = XmStringCreateLtoR ( "User Interface...", (XmStringCharSet)XmFONTLIST_DEFAULT_TAG );
	XtSetArg(al[ac], XmNlabelString, xmstrings[0]); ac++;
	button26 = XmCreatePushButton ( menu3, "MetviewUI", al, ac );
	ac = 0;
	XmStringFree ( xmstrings [ 0 ] );
	xmstrings[0] = XmStringCreateLtoR ( "Macro language...", (XmStringCharSet)XmFONTLIST_DEFAULT_TAG );
	XtSetArg(al[ac], XmNlabelString, xmstrings[0]); ac++;
	button27 = XmCreatePushButton ( menu3, "Macro", al, ac );
	ac = 0;
	XmStringFree ( xmstrings [ 0 ] );
	xmstrings[0] = XmStringCreateLtoR ( "Applications...", (XmStringCharSet)XmFONTLIST_DEFAULT_TAG );
	XtSetArg(al[ac], XmNlabelString, xmstrings[0]); ac++;
	button28 = XmCreatePushButton ( menu3, "applications", al, ac );
	ac = 0;
	XmStringFree ( xmstrings [ 0 ] );
	xmstrings[0] = XmStringCreateLtoR ( "Plot Module...", (XmStringCharSet)XmFONTLIST_DEFAULT_TAG );
	XtSetArg(al[ac], XmNlabelString, xmstrings[0]); ac++;
	button31 = XmCreatePushButton ( menu3, "PlotMod", al, ac );
	ac = 0;
	XmStringFree ( xmstrings [ 0 ] );
	separator13 = XmCreateSeparator ( menu3, "separator13", al, ac );
	XtSetArg(al[ac], XmNinitialResourcesPersistent, FALSE); ac++;
	xmstrings[0] = XmStringCreateLtoR ( "Support...", (XmStringCharSet)XmFONTLIST_DEFAULT_TAG );
	XtSetArg(al[ac], XmNlabelString, xmstrings[0]); ac++;
	button29 = XmCreatePushButton ( menu3, "support", al, ac );
	ac = 0;
	XmStringFree ( xmstrings [ 0 ] );
	XtSetArg(al[ac], XmNinitialResourcesPersistent, FALSE); ac++;
	xmstrings[0] = XmStringCreateLtoR ( "Updates...", (XmStringCharSet)XmFONTLIST_DEFAULT_TAG );
	XtSetArg(al[ac], XmNlabelString, xmstrings[0]); ac++;
	button30 = XmCreatePushButton ( menu3, "updates", al, ac );
	ac = 0;
	XmStringFree ( xmstrings [ 0 ] );
	XtSetArg(al[ac], XmNmenuHelpWidget, help_); ac++;
	XtSetValues ( menuBar1,al, ac );
	ac = 0;

	XtSetArg(al[ac], XmNtopAttachment, XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNtopWidget, menuBar1); ac++;
	XtSetArg(al[ac], XmNbottomAttachment, XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNbottomWidget, drawer_); ac++;
	XtSetArg(al[ac], XmNleftAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightAttachment, XmATTACH_FORM); ac++;
	XtSetValues ( scrolledWin1,al, ac );
	ac = 0;

	XtSetArg(al[ac], XmNtopAttachment, XmATTACH_NONE); ac++;
	XtSetArg(al[ac], XmNbottomAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNleftAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightAttachment, XmATTACH_FORM); ac++;
	XtSetValues ( drawer_,al, ac );
	ac = 0;

	XtSetArg(al[ac], XmNtopAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomAttachment, XmATTACH_NONE); ac++;
	XtSetArg(al[ac], XmNleftAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightAttachment, XmATTACH_FORM); ac++;
	XtSetValues ( menuBar1,al, ac );
	ac = 0;
	XtAddCallback (commands_, XmNentryCallback, commandCB, (XtPointer) this);
	children[ac++] = label1;
	children[ac++] = separator2;
	children[ac++] = button10;
	children[ac++] = button11;
	children[ac++] = button13;
	children[ac++] = button14;
	children[ac++] = button101;
	children[ac++] = separator5;
	children[ac++] = button12;
	children[ac++] = button15;
	children[ac++] = button16;
	children[ac++] = button6;
	children[ac++] = separator4;
	children[ac++] = button5;
	XtManageChildren(children, ac);
	ac = 0;
	XtAddCallback (button8, XmNactivateCallback, newIconCB, (XtPointer) this);
	XtAddCallback (button9, XmNactivateCallback, newLinkCB, (XtPointer) this);
	XtAddCallback (button22, XmNactivateCallback, sortByNameCB, (XtPointer) this);
	XtAddCallback (button24, XmNactivateCallback, sortByClassCB, (XtPointer) this);
	XtAddCallback (button20, XmNactivateCallback, rescanCB, (XtPointer) this);
	XtAddCallback (button21, XmNactivateCallback, toggleSizeCB, (XtPointer) this);
	XtAddCallback (button17, XmNactivateCallback, showLastCB, (XtPointer) this);
	children[ac++] = button8;
	children[ac++] = button9;
	children[ac++] = separator7;
	children[ac++] = button22;
	children[ac++] = button24;
	children[ac++] = separator10;
	children[ac++] = button20;
	children[ac++] = separator9;
	children[ac++] = button21;
	children[ac++] = button17;
	XtManageChildren(children, ac);
	ac = 0;
	children[ac++] = drag_;
	XtManageChildren(children, ac);
	ac = 0;
	XmScrolledWindowSetAreas(scrolledWin1, scrollbar1, scrollbar2, drag_ );
	XtAddCallback (menu1, XmNmapCallback, fileMapCB, (XtPointer) this);
	XtAddCallback (button18, XmNactivateCallback, prefCB, (XtPointer) this);
	XtAddCallback (button23, XmNactivateCallback, messagesCB, (XtPointer) this);
	XtAddCallback (close_, XmNactivateCallback, closeCB, (XtPointer) this);
	XtAddCallback (button2, XmNactivateCallback, exitCB, (XtPointer) this);
	children[ac++] = button18;
	children[ac++] = separator8;
	children[ac++] = button23;
	children[ac++] = separator11;
	children[ac++] = close_;
	children[ac++] = button2;
	XtManageChildren(children, ac);
	ac = 0;
	XtSetArg(al[ac], XmNsubMenuId, menu1); ac++;
	XtSetValues ( cascade1, al, ac );
	ac = 0;
	XtAddCallback (button1, XmNactivateCallback, sortByNameCB, (XtPointer) this);
	XtAddCallback (button3, XmNactivateCallback, sortByClassCB, (XtPointer) this);
	XtAddCallback (button4, XmNactivateCallback, toggleSizeCB, (XtPointer) this);
	XtAddCallback (button19, XmNactivateCallback, rescanCB, (XtPointer) this);
	XtAddCallback (button7, XmNactivateCallback, showLastCB, (XtPointer) this);
	children[ac++] = button1;
	children[ac++] = button3;
	children[ac++] = separator3;
	children[ac++] = button4;
	children[ac++] = button19;
	children[ac++] = separator6;
	children[ac++] = button7;
	XtManageChildren(children, ac);
	ac = 0;
	XtSetArg(al[ac], XmNsubMenuId, menu2); ac++;
	XtSetValues ( cascade3, al, ac );
	ac = 0;
	XtSetArg(al[ac], XmNsubMenuId, tools_); ac++;
	XtSetValues ( cascade5, al, ac );
	ac = 0;
	XtAddCallback (recent_, XmNentryCallback, recentCB, (XtPointer) this);
	children[ac++] = top_;
	children[ac++] = up_;
	children[ac++] = separator1;
	XtManageChildren(children, ac);
	ac = 0;
	XtSetArg(al[ac], XmNsubMenuId, recent_); ac++;
	XtSetValues ( cascade4, al, ac );
	ac = 0;
	XtAddCallback (menu3, XmNentryCallback, helpMenuCB, (XtPointer) this);
	children[ac++] = button25;
	children[ac++] = separator12;
	children[ac++] = button26;
	children[ac++] = button27;
	children[ac++] = button28;
	children[ac++] = button31;
	children[ac++] = separator13;
	children[ac++] = button29;
	children[ac++] = button30;
	XtManageChildren(children, ac);
	ac = 0;
	XtSetArg(al[ac], XmNsubMenuId, menu3); ac++;
	XtSetValues ( help_, al, ac );
	ac = 0;
	children[ac++] = cascade1;
	children[ac++] = cascade3;
	children[ac++] = cascade5;
	children[ac++] = cascade4;
	children[ac++] = help_;
	XtManageChildren(children, ac);
	ac = 0;
	children[ac++] = scrolledWin1;
	children[ac++] = drawer_;
	children[ac++] = menuBar1;
	XtManageChildren(children, ac);
	ac = 0;
	XtManageChild ( form_);
}

void UIFolderWindow_c::helpMenuCB( Widget widget, XtPointer client_data, XtPointer call_data )
{
	UIFolderWindow_p instance = (UIFolderWindow_p) client_data;
	instance->helpMenuCB ( widget, call_data );
}

void UIFolderWindow_c::messagesCB( Widget widget, XtPointer client_data, XtPointer call_data )
{
	UIFolderWindow_p instance = (UIFolderWindow_p) client_data;
	instance->messagesCB ( widget, call_data );
}

void UIFolderWindow_c::rescanCB( Widget widget, XtPointer client_data, XtPointer call_data )
{
	UIFolderWindow_p instance = (UIFolderWindow_p) client_data;
	instance->rescanCB ( widget, call_data );
}

void UIFolderWindow_c::toggleSizeCB( Widget widget, XtPointer client_data, XtPointer call_data )
{
	UIFolderWindow_p instance = (UIFolderWindow_p) client_data;
	instance->toggleSizeCB ( widget, call_data );
}

void UIFolderWindow_c::prefCB( Widget widget, XtPointer client_data, XtPointer call_data )
{
	UIFolderWindow_p instance = (UIFolderWindow_p) client_data;
	instance->prefCB ( widget, call_data );
}

void UIFolderWindow_c::newLinkCB( Widget widget, XtPointer client_data, XtPointer call_data )
{
	UIFolderWindow_p instance = (UIFolderWindow_p) client_data;
	instance->newLinkCB ( widget, call_data );
}

void UIFolderWindow_c::newIconCB( Widget widget, XtPointer client_data, XtPointer call_data )
{
	UIFolderWindow_p instance = (UIFolderWindow_p) client_data;
	instance->newIconCB ( widget, call_data );
}

void UIFolderWindow_c::recentCB( Widget widget, XtPointer client_data, XtPointer call_data )
{
	UIFolderWindow_p instance = (UIFolderWindow_p) client_data;
	instance->recentCB ( widget, call_data );
}

void UIFolderWindow_c::showLastCB( Widget widget, XtPointer client_data, XtPointer call_data )
{
	UIFolderWindow_p instance = (UIFolderWindow_p) client_data;
	instance->showLastCB ( widget, call_data );
}

void UIFolderWindow_c::sortByClassCB( Widget widget, XtPointer client_data, XtPointer call_data )
{
	UIFolderWindow_p instance = (UIFolderWindow_p) client_data;
	instance->sortByClassCB ( widget, call_data );
}

void UIFolderWindow_c::sortByNameCB( Widget widget, XtPointer client_data, XtPointer call_data )
{
	UIFolderWindow_p instance = (UIFolderWindow_p) client_data;
	instance->sortByNameCB ( widget, call_data );
}

void UIFolderWindow_c::fileMapCB( Widget widget, XtPointer client_data, XtPointer call_data )
{
	UIFolderWindow_p instance = (UIFolderWindow_p) client_data;
	instance->fileMapCB ( widget, call_data );
}

void UIFolderWindow_c::commandCB( Widget widget, XtPointer client_data, XtPointer call_data )
{
	UIFolderWindow_p instance = (UIFolderWindow_p) client_data;
	instance->commandCB ( widget, call_data );
}

void UIFolderWindow_c::exitCB( Widget widget, XtPointer client_data, XtPointer call_data )
{
	UIFolderWindow_p instance = (UIFolderWindow_p) client_data;
	instance->exitCB ( widget, call_data );
}

void UIFolderWindow_c::closeCB( Widget widget, XtPointer client_data, XtPointer call_data )
{
	UIFolderWindow_p instance = (UIFolderWindow_p) client_data;
	instance->closeCB ( widget, call_data );
}

