/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

/*
** Generated by X-Designer
*/
#ifndef _UISyncFolders_h
#define _UISyncFolders_h

#define XD_MOTIF

#include <xdclass.h>

class UISyncFolders_c: public xd_ApplicationShell_c {
public:
	virtual void create (Display *display, char *app_name, int app_argc, char **app_argv, char *app_class_name = NULL);
protected:
	Widget UISyncFolders;
	Widget text_;
	Widget added_form_;
	Widget add_all_;
	Widget add_selected_;
	Widget add_none_;
	Widget added_;
	Widget changed_form_;
	Widget preserve_all_;
	Widget preserve_selected_;
	Widget overwrite_all_;
	Widget overwrite_selected_;
	Widget changed_u_;
	Widget changed_s_;
	Widget gone_form_;
	Widget remove_all_;
	Widget remove_selected_;
	Widget keep_all_;
	Widget gone_;
public:
	static void selectCB( Widget, XtPointer, XtPointer );
	virtual void selectCB( Widget, XtPointer ) = 0;
	static void entryCB( Widget, XtPointer, XtPointer );
	virtual void entryCB( Widget, XtPointer ) = 0;
	static void syncCB( Widget, XtPointer, XtPointer );
	virtual void syncCB( Widget, XtPointer ) = 0;
	static void closeCB( Widget, XtPointer, XtPointer );
	virtual void closeCB( Widget, XtPointer ) = 0;
};

typedef UISyncFolders_c *UISyncFolders_p;


extern UISyncFolders_p UISyncFolders;


#endif
