/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

// File UserMesssage.h
// Geir Austad - ECMWF May 01

#ifndef UserMessage_H
#define UserMessage_H


#ifndef  Message_H
#include <Message.h>
#endif

class UserMessage : public Message
{
  // -- From MvMessage
  virtual void callback(MvRequest&);

 public:
  UserMessage(); 
  ~UserMessage();
	
 private:

  // No copy allowed
  
  UserMessage(const UserMessage&);
  UserMessage& operator=(const UserMessage&);
};



inline void destroy(UserMessage**) {}

#endif
