/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

// File TemporaryFolder.h
// Baudouin Raoult - ECMWF Jul 99

#ifndef TemporaryFolder_H
#define TemporaryFolder_H

#include "SystemFolder.h"

class TemporaryFolder : public SystemFolder {
public:

// -- Exceptions
	// None

// -- Contructors

	TemporaryFolder(Folder* parent,const IconClass& kind,
		const string& name,IconInfo* info);

// -- Destructor

	virtual ~TemporaryFolder(); // Change to virtual if base class

// -- Convertors
	// None

// -- Operators
	// None

// -- Methods
	// None

// -- Overridden methods
	// None

// -- Class members
	// None

// -- Class methods

protected:

// -- Members
	// None

// -- Methods
	
	// void print(ostream&) const; // Change to virtual if base class	

// -- Overridden methods
	// None

// -- Class members
	// None

// -- Class methods
	// None

private:

// No copy allowed

	TemporaryFolder(const TemporaryFolder&);
	TemporaryFolder& operator=(const TemporaryFolder&);

// -- Members

// -- Methods
	// None

// -- Overridden methods

	virtual void createFiles();

// -- Class members
	// None

// -- Class methods
	// None

// -- Friends

	//friend ostream& operator<<(ostream& s,const TemporaryFolder& p)
	//	{ p.print(s); return s; }

};

inline void destroy(TemporaryFolder**) {}

// If persistent, uncomment, otherwise remove
//#ifdef _ODI_OSSG_
//OS_MARK_SCHEMA_TYPE(TemporaryFolder);
//#endif

#endif
