/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

// File NoEditor.h
// Baudouin Raoult - ECMWF Jul 99

#ifndef NoEditor_H
#define NoEditor_H

#ifndef InternalEditor_H
#include "InternalEditor.h"
#endif

#include "UINoEditor.h"

class NoEditor : public XEditor<UINoEditor_c> {
public:

// -- Exceptions
	// None

// -- Contructors

	NoEditor(const IconClass&,const string&);

// -- Destructor

	virtual ~NoEditor(); // Change to virtual if base class

// -- Convertors
	// None

// -- Operators
	// None

// -- Methods
	// None

// -- Overridden methods
	// None

// -- Class members
	// None

// -- Class methods
	// None

	// Uncomment for persistent, remove otherwise
	// static os_typespec* get_os_typespec();

protected:

// -- Members
	// None

// -- Methods
	
	// void print(ostream&) const; // Change to virtual if base class	

// -- Overridden methods
	// None

// -- Class members
	// None

// -- Class methods
	// None

private:

// No copy allowed

	NoEditor(const NoEditor&);
	NoEditor& operator=(const NoEditor&);

// -- Members
	// None

// -- Methods
	// None

// -- Overridden methods

	virtual void apply();
	virtual void reset();
	virtual void close();
	virtual void merge(IconObject*);
	virtual void replace(IconObject*);
	virtual string alternateEditor();

// -- Class members
	// None

// -- Class methods
	// None

// -- Friends

	//friend ostream& operator<<(ostream& s,const NoEditor& p)
	//	{ p.print(s); return s; }

};

inline void destroy(NoEditor**) {}

// If persistent, uncomment, otherwise remove
//#ifdef _ODI_OSSG_
//OS_MARK_SCHEMA_TYPE(NoEditor);
//#endif

#endif
