/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

/*
** Generated by X-Designer
*/
#ifndef _UIIconHelp_h
#define _UIIconHelp_h

#define XD_MOTIF

#include <xdclass.h>

class UIIconHelp_c: public xd_XmForm_c {
public:
	virtual void create (Widget parent, char *widget_name = NULL);
protected:
	Widget UIIconHelp;
	Widget drag_;
	Widget menu1_;
	Widget menu2_;
	Widget help_;
public:
	static void newIconCB( Widget, XtPointer, XtPointer );
	virtual void newIconCB( Widget, XtPointer ) = 0;
	static void deleteCB( Widget, XtPointer, XtPointer );
	virtual void deleteCB( Widget, XtPointer ) = 0;
	static void editCB( Widget, XtPointer, XtPointer );
	virtual void editCB( Widget, XtPointer ) = 0;
	static void folderCB( Widget, XtPointer, XtPointer );
	virtual void folderCB( Widget, XtPointer ) = 0;
};

typedef UIIconHelp_c *UIIconHelp_p;


extern UIIconHelp_p UIIconHelp;


#endif
