/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

#ifndef VerbFunctionHelpDialog_H
#define VerbFunctionHelpDialog_H


#include <vector>
#include <map>
using namespace std;


#include "ui_VerbFunctionHelpDialog.h"
#include "VerbFunctionHelp.h"



// VerbFunctionHelpDialog
// the dialog class itself 

class VerbFunctionHelpDialog : public QDialog, private Ui::VerbFunctionHelpDialog
{
    Q_OBJECT

public:
    VerbFunctionHelpDialog();
    ~VerbFunctionHelpDialog(){};

    void helpForFunction(VerbFunctionHelp *help, QString &functionName);


signals:
    void sendTextToBeInsertedIntoFunctionCall (QString text);


public slots:
    void onParameterSelected (QTreeWidgetItem *item,  QTreeWidgetItem  *previous);
    void onInsertButtonClicked ();


private:
    VerbFunctionHelpFunction *currentHelpFunction;
};



#endif

