/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

/*
** Generated by X-Designer
*/
#ifndef _UIViewEditor_h
#define _UIViewEditor_h

#define XD_MOTIF

#include <xdclass.h>

class UIViewEditor_c: public xd_XmForm_c {
public:
	virtual void create (Widget parent, char *widget_name = NULL);
protected:
	Widget UIViewEditor;
	Widget paper_;
public:
	static void joinCB( Widget, XtPointer, XtPointer );
	virtual void joinCB( Widget, XtPointer ) = 0;
	static void groupCB( Widget, XtPointer, XtPointer );
	virtual void groupCB( Widget, XtPointer ) = 0;
	static void ungroupCB( Widget, XtPointer, XtPointer );
	virtual void ungroupCB( Widget, XtPointer ) = 0;
	static void deleteCB( Widget, XtPointer, XtPointer );
	virtual void deleteCB( Widget, XtPointer ) = 0;
	static void splitCB( Widget, XtPointer, XtPointer );
	virtual void splitCB( Widget, XtPointer ) = 0;
	static void changeCB( Widget, XtPointer, XtPointer );
	virtual void changeCB( Widget, XtPointer ) = 0;
	static void drawCB( Widget, XtPointer, XtPointer );
	virtual void drawCB( Widget, XtPointer ) = 0;
	static void dblClickCB( Widget, XtPointer, XtPointer );
	virtual void dblClickCB( Widget, XtPointer ) = 0;
};

typedef UIViewEditor_c *UIViewEditor_p;


extern UIViewEditor_p UIViewEditor;


#endif
