/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

/*
** Generated by X-Designer
*/
#ifndef _UIODBFunctionsDrawer_h
#define _UIODBFunctionsDrawer_h

#define XD_MOTIF

#include <xdclass.h>

class UIODBFunctionsDrawer_c: public xd_XmForm_c {
public:
	virtual void create (Widget parent, char *widget_name = NULL);
protected:
	Widget UIODBFunctionsDrawer;
	Widget operators_;
	Widget functions_;
	Widget contants_;
	Widget help_;
public:
	static void browseCB( Widget, XtPointer, XtPointer );
	virtual void browseCB( Widget, XtPointer ) = 0;
	static void doubleClickCB( Widget, XtPointer, XtPointer );
	virtual void doubleClickCB( Widget, XtPointer ) = 0;
};

typedef UIODBFunctionsDrawer_c *UIODBFunctionsDrawer_p;


extern UIODBFunctionsDrawer_p UIODBFunctionsDrawer;


#endif
