/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

// File Dependancy.h
// Baudouin Raoult - ECMWF Aug 99

#ifndef Dependancy_H
#define Dependancy_H

// Headers
// #ifndef   machine_H
// #include <machine.h>
// #endif

// Forward declarations

// class ostream;
// typedef class _Pvts os_typespec; // Remove if not persistant

// 

#ifndef Counted_H
#include "Counted.h"
#endif

class IconObject;
class Task;
class Action;
class Request;


class Dependancy : public virtual Counted {
public:

// -- Exceptions
	// None

// -- Contructors

	Dependancy();

// -- Destructor

	virtual ~Dependancy(); // Change to virtual if base class

// -- Convertors
	// None

// -- Operators


// -- Methods

	virtual Task* action(const Action&)   = 0;
	virtual void  success(const Request&) = 0;
	virtual void  failure() =0;

// -- Overridden methods
	// None

// -- Class members
	// None

// -- Class methods
	// None

	// Uncomment for persistent, remove otherwise
	// static os_typespec* get_os_typespec();

protected:

// -- Members
	// None

// -- Methods
	
	// void print(ostream&) const; // Change to virtual if base class	

// -- Overridden methods
	// None

// -- Class members
	// None

// -- Class methods
	// None

private:

// No copy allowed


// -- Members


// -- Methods
	// None

// -- Overridden methods
	// None

// -- Class members
	// None

// -- Class methods
	// None

// -- Friends

	//friend ostream& operator<<(ostream& s,const Dependancy& p)
	//	{ p.print(s); return s; }

};

class DependancyH : public Handle<Dependancy> {
public:
	DependancyH(Dependancy* o = 0) : Handle<Dependancy>(o) {}
};

#endif
