/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

// File LayoutDrawer.h
// Baudouin Raoult - ECMWF Jul 99

#ifndef LayoutDrawer_H
#define LayoutDrawer_H

#ifndef EditorDrawer_H
#include "EditorDrawer.h"
#endif

#include "UILayoutDrawer.h"

class ViewEditor;

class LayoutDrawer : public XEditorDrawer<UILayoutDrawer_c> {
public:

// -- Exceptions
	// None

// -- Contructors

	LayoutDrawer(ViewEditor&);

// -- Destructor

	virtual ~LayoutDrawer(); // Change to virtual if base class

// -- Convertors
	// None

// -- Operators
	// None

// -- Methods
	// None

// -- Overridden methods

// -- Class members
	// None

// -- Class methods
	// None

	// Uncomment for persistent, remove otherwise
	// static os_typespec* get_os_typespec();

protected:

// -- Members
	// None

// -- Methods
	
	// void print(ostream&) const; // Change to virtual if base class	

// -- Overridden methods
	// None

// -- Class members
	// None

// -- Class methods
	// None

private:

// No copy allowed

	LayoutDrawer(const LayoutDrawer&);
	LayoutDrawer& operator=(const LayoutDrawer&);

// -- Members

  	ViewEditor& editor_;

// -- Methods
	// None

// -- Overridden methods

	virtual void update();

	void splitVerCB(Widget, XtPointer);
	void splitHorCB(Widget, XtPointer);
	void groupSelectionCB(Widget, XtPointer);
	void ungroupSelectionCB(Widget, XtPointer);
	void joinSelectionCB(Widget, XtPointer);
	void deleteSelectionCB(Widget, XtPointer);
	void addPageCB(Widget, XtPointer);

// -- Class members
	// None

// -- Class methods
	// None

// -- Friends

	//friend ostream& operator<<(ostream& s,const LayoutDrawer& p)
	//	{ p.print(s); return s; }

};

inline void destroy(LayoutDrawer**) {}

// If persistent, uncomment, otherwise remove
//#ifdef _ODI_OSSG_
//OS_MARK_SCHEMA_TYPE(LayoutDrawer);
//#endif

#endif
