/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

// File Input.h
// Baudouin Raoult - ECMWF Oct 98

#ifndef Input_H
#define Input_H

#include <Xm/Xm.h>

// Headers
// #ifndef   machine_H
// #include <machine.h>
// #endif

// Forward declarations

// class ostream;
// typedef class _Pvts os_typespec; // Remove if not persistant


// 

#include <stdio.h>

class Input {
public:

// -- Exceptions
	// None

// -- Contructors

	Input();

// -- Destructor

	virtual ~Input(); // Change to virtual if base class

// -- Convertors
	// None

// -- Operators
	// None

// -- Methods
	// None

	void start(FILE*);
	void stop();

	virtual void ready(const char*) = 0;
	virtual void done(FILE*)        = 0;

// -- Overridden methods
	// None

// -- Class members
	// None

// -- Class methods
	// None

	// Uncomment for persistent, remove otherwise
	// static os_typespec* get_os_typespec();

protected:

// -- Members


// -- Methods
	
	// void print(ostream&) const; // Change to virtual if base class	

// -- Overridden methods
	// None

// -- Class members
	// None

// -- Class methods
	// None

private:

// No copy allowed

	Input(const Input&);
	Input& operator=(const Input&);

// -- Members

	XtInputId  id_;
	FILE*       file_;

// -- Methods
	// None

// -- Overridden methods
	// None

// -- Class members

// -- Class methods
	// None
	static void inputCB(XtPointer,int*,XtInputId*);

// -- Friends

	//friend ostream& operator<<(ostream& s,const Input& p)
	//	{ p.print(s); return s; }

};


#endif
