/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

/*
** Generated by X-Designer
*/
#ifndef _UIIconDrawer_h
#define _UIIconDrawer_h

#define XD_MOTIF

#include <xdclass.h>

class UIIconDrawer_c: public xd_XmForm_c {
public:
	virtual void create (Widget parent, char *widget_name = NULL);
protected:
	Widget UIIconDrawer;
	Widget drag_;
	Widget menu1_;
	Widget menu2_;
public:
	static void updateDrawerCB( Widget, XtPointer, XtPointer );
	virtual void updateDrawerCB( Widget, XtPointer ) = 0;
	static void openFolderCB( Widget, XtPointer, XtPointer );
	virtual void openFolderCB( Widget, XtPointer ) = 0;
	static void deleteDrawerCB( Widget, XtPointer, XtPointer );
	virtual void deleteDrawerCB( Widget, XtPointer ) = 0;
	static void renameDrawerCB( Widget, XtPointer, XtPointer );
	virtual void renameDrawerCB( Widget, XtPointer ) = 0;
	static void newDrawerCB( Widget, XtPointer, XtPointer );
	virtual void newDrawerCB( Widget, XtPointer ) = 0;
	static void newIconCB( Widget, XtPointer, XtPointer );
	virtual void newIconCB( Widget, XtPointer ) = 0;
	static void deleteCB( Widget, XtPointer, XtPointer );
	virtual void deleteCB( Widget, XtPointer ) = 0;
	static void editCB( Widget, XtPointer, XtPointer );
	virtual void editCB( Widget, XtPointer ) = 0;
};

typedef UIIconDrawer_c *UIIconDrawer_p;


extern UIIconDrawer_p UIIconDrawer;


#endif
