/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

// File TextLine.h
// Baudouin Raoult - ECMWF Jul 99

#ifndef ExternalHelp_H
#define ExternalHelp_H

#ifndef RequestPanelHelp_H
#include "RequestPanelHelp.h"
#endif

#ifndef Request_H
#include "Request.h"
#endif

#ifndef ReplyObserver_H
#include "ReplyObserver.h"
#endif

class ExternalHelp: public DialogHelp, public ReplyObserver {
public:

// -- Contructors

	ExternalHelp(Widget, RequestPanel&, const Parameter&);

// -- Destructor

	virtual ~ExternalHelp(); 

private:

// No copy allowed

	ExternalHelp(const ExternalHelp&);
	ExternalHelp& operator=(const ExternalHelp&);

// -- Members
	
	Request request_;


// -- Overridden methods

	// From DialogHelp

	virtual void start();

	// From ReplyObserver 

	virtual void reply(const Request&, int);
	virtual void progress(const Request&);
        virtual void message(const string& );

	// From RequestPanelItem
                  
	virtual void set(Request&);
       	
};

#endif
