/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

// File NoneLine.h
// Baudouin Raoult - ECMWF Jul 99

#ifndef NoneLine_H
#define NoneLine_H

#include "UINoneLine.h"
#include "RequestPanelLine.h"


class NoneLine : public XRequestPanelLine<UINoneLine_c> {
public:

// -- Contructors

	NoneLine(RequestPanel&,const Parameter&);

// -- Destructor

	virtual ~NoneLine(); 

private:

// No copy allowed

	NoneLine(const NoneLine&);
	NoneLine& operator=(const NoneLine&);

// -- Overridden methods
	
	// From RequestPanelLine

	virtual void init(Widget);
	virtual void gray(bool);

};

#endif
