/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

/*
** Generated by X-Designer
*/
#ifndef _UIODBEditor_h
#define _UIODBEditor_h

#define XD_MOTIF

#include <xdclass.h>

class UIODBEditor_c: public xd_XmForm_c {
public:
	virtual void create (Widget parent, char *widget_name = NULL);
protected:
	Widget UIODBEditor;
	Widget selectList_;
	Widget add_;
	Widget remove_;
	Widget up_;
	Widget down_;
	Widget whereText_;
	Widget insert_;
	Widget tablesList_;
	Widget columnsList_;
	Widget fieldsList_;
public:
	static void doubleClickCB( Widget, XtPointer, XtPointer );
	virtual void doubleClickCB( Widget, XtPointer ) = 0;
	static void changedCB( Widget, XtPointer, XtPointer );
	virtual void changedCB( Widget, XtPointer ) = 0;
	static void downCB( Widget, XtPointer, XtPointer );
	virtual void downCB( Widget, XtPointer ) = 0;
	static void upCB( Widget, XtPointer, XtPointer );
	virtual void upCB( Widget, XtPointer ) = 0;
	static void insertCB( Widget, XtPointer, XtPointer );
	virtual void insertCB( Widget, XtPointer ) = 0;
	static void removeCB( Widget, XtPointer, XtPointer );
	virtual void removeCB( Widget, XtPointer ) = 0;
	static void addCB( Widget, XtPointer, XtPointer );
	virtual void addCB( Widget, XtPointer ) = 0;
	static void browseSelectCB( Widget, XtPointer, XtPointer );
	virtual void browseSelectCB( Widget, XtPointer ) = 0;
	static void browseFieldsCB( Widget, XtPointer, XtPointer );
	virtual void browseFieldsCB( Widget, XtPointer ) = 0;
	static void browseColumnsCB( Widget, XtPointer, XtPointer );
	virtual void browseColumnsCB( Widget, XtPointer ) = 0;
	static void browseTablesCB( Widget, XtPointer, XtPointer );
	virtual void browseTablesCB( Widget, XtPointer ) = 0;
};

typedef UIODBEditor_c *UIODBEditor_p;


extern UIODBEditor_p UIODBEditor;


#endif
