/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

#ifndef CAROUSEL_H
#define CAROUSEL_H

#define XtNspeed   "speed"
#define XtCSpeed   "Speed"

extern WidgetClass  carouselWidgetClass;

typedef struct _CarouselClassRec *CarouselWidgetClass;
typedef struct _CarouselRec      *CarouselWidget;
#ifdef _NO_PROTO
 
extern Widget CreateCarousel();
 
#else
 
#if defined(__cplusplus) || defined(c_plusplus)
extern "C" {
#endif
 
Widget    CreateCarousel(Widget,String,Arg*,int);

#if defined(__cplusplus) || defined(c_plusplus)
}
#endif
 
#endif /* _NO_PROTO */
 


#endif /* CAROUSEL_H */
