/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

// File MenuLine.h
// Baudouin Raoult - ECMWF Jul 99

#ifndef MenuLine_H
#define MenuLine_H

#include "UIMenuLine.h"
#include "RequestPanelLine.h"


class MenuLine : public XRequestPanelLine<UIMenuLine_c> {
public:

// -- Contructors

	MenuLine(RequestPanel&,const Parameter&);

// -- Destructor

	virtual ~MenuLine(); 

private:

// No copy allowed

	MenuLine(const MenuLine&);
	MenuLine& operator=(const MenuLine&);

// -- Overridden methods

	// From XD

	virtual void activateCB(Widget,XtPointer);

	// From RequestPanelItem

	virtual void refresh(const vector<string>&);
	
	// From RequestPanelLine

	virtual void init(Widget);

};

#endif
