/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

// File Transaction.h
// Baudouin Raoult - ECMWF Jul 99

#ifndef Transaction_H
#define Transaction_H

// Headers
// #ifndef   machine_H
// #include <machine.h>
// #endif

// Forward declarations

// class ostream;
// typedef class _Pvts os_typespec; // Remove if not persistant

// 
#ifndef Metview_H
#include <XMLabel.h>
#include <Metview.h>
#define Metview_H
#endif

class Transaction : public MvTransaction {
public:

// -- Exceptions
	// None

// -- Contructors

	Transaction(const char*);
	Transaction(MvTransaction*);

// -- Destructor

	~Transaction(); // Change to virtual if base class

// -- Convertors
	// None

// -- Operators
	// None

// -- Methods
	// None

// -- Overridden methods
	// None
	virtual MvTransaction *cloneSelf() = 0;
	virtual void callback(MvRequest&)  = 0;


// -- Class members
	// None

// -- Class methods

	// Uncomment for persistent, remove otherwise
	// static os_typespec* get_os_typespec();

protected:

// -- Members
	// None

// -- Methods
	
	// void print(ostream&) const; // Change to virtual if base class	

// -- Overridden methods
	// None

// -- Class members
	// None

// -- Class methods
	// None

private:

// No copy allowed

	Transaction(const Transaction&);
	Transaction& operator=(const Transaction&);

// -- Members
	// None

// -- Methods
	// None

// -- Overridden methods
	// None

// -- Class members
	// None

// -- Class methods
	// None

// -- Friends

	//friend ostream& operator<<(ostream& s,const Transaction& p)
	//	{ p.print(s); return s; }

};

inline void destroy(Transaction**) {}

// If persistent, uncomment, otherwise remove
//#ifdef _ODI_OSSG_
//OS_MARK_SCHEMA_TYPE(Transaction);
//#endif

#endif
