/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

// File FileObject.h
// Baudouin Raoult - ECMWF Jul 99

#ifndef FileObject_H
#define FileObject_H

#ifndef IconObject_H
#include "IconObject.h"
#endif


#include <string>
using std::string;


class FileObject : public IconObject {
public:

// -- Exceptions
	// None

// -- Contructors

	FileObject(Folder* parent,
		const IconClass& kind, const string& name,
		IconInfo* info);

// -- Destructor

	virtual ~FileObject(); // Change to virtual if base class

// -- Convertors
	// None

// -- Operators
	// None

// -- Methods


// -- Overridden methods
	// None

// -- Class members
	// None

// -- Class methods
	// None

	// Uncomment for persistent, remove otherwise
	// static os_typespec* get_os_typespec();

protected:

// -- Members
	// None

// -- Methods
	
	// void print(ostream&) const; // Change to virtual if base class	

// -- Overridden methods
	// None

// -- Class members
	// None

// -- Class methods
	// None

private:

// No copy allowed

	FileObject(const FileObject&);
	FileObject& operator=(const FileObject&);

// -- Members

// -- Methods
	// None

// -- Overridden methods

	// From IconObject

	virtual void doubleClick();
	virtual set<string> can();

	virtual void createFiles();
	virtual Request request() const;
	virtual void request(const Request&);

// -- Class members
	// None

// -- Class methods
	// None

// -- Friends

	//friend ostream& operator<<(ostream& s,const FileObject& p)
	//	{ p.print(s); return s; }

};

inline void destroy(FileObject**) {}

// If persistent, uncomment, otherwise remove
//#ifdef _ODI_OSSG_
//OS_MARK_SCHEMA_TYPE(FileObject);
//#endif

#endif
