/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

/*
** Generated by X-Designer
*/
#ifndef _UIColourEditor_h
#define _UIColourEditor_h

#define XD_MOTIF

#include <xdclass.h>

class UIColourEditor_c: public xd_XmForm_c {
public:
	virtual void create (Widget parent, char *widget_name = NULL);
protected:
	Widget UIColourEditor;
	Widget old_;
	Widget new_;
	Widget tab_;
	Widget names_;
	Widget r_;
	Widget rr_;
	Widget g_;
	Widget gg_;
	Widget b_;
	Widget bb_;
	Widget h_;
	Widget hh_;
	Widget s_;
	Widget ss_;
	Widget l_;
	Widget ll_;
public:
	static void tabCB( Widget, XtPointer, XtPointer );
	virtual void tabCB( Widget, XtPointer ) = 0;
	static void exposeCB( Widget, XtPointer, XtPointer );
	virtual void exposeCB( Widget, XtPointer ) = 0;
	static void hslCB( Widget, XtPointer, XtPointer );
	virtual void hslCB( Widget, XtPointer ) = 0;
	static void rgbCB( Widget, XtPointer, XtPointer );
	virtual void rgbCB( Widget, XtPointer ) = 0;
};

typedef UIColourEditor_c *UIColourEditor_p;


extern UIColourEditor_p UIColourEditor;


#endif
