#ifndef marray_DEFINED
#define marray_DEFINED

#include <MMotifOnly.hpp>
#include <Array.h>

class MArray :  public MWidget
{
public:
		void    init(MWidget *parent, const char *name, ...);
};

#endif
