/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

/*
** Generated by X-Designer
*/
/*
**LIBS: -lXm -lXt -lX11
*/

#include <stdlib.h>
#include <X11/Xatom.h>
#include <X11/Intrinsic.h>
#include <X11/Shell.h>

#include <Xm/Xm.h>
#include <Xm/DialogS.h>
#include <Xm/Form.h>
#include <Xm/TextF.h>

#include <xdclass.h>

class UITextLine_c: public xd_XmForm_c {
public:
	virtual void create (Widget parent, char *widget_name = NULL);
protected:
	Widget UITextLine;
	Widget text_;
public:
	static void changedCB( Widget, XtPointer, XtPointer );
	virtual void changedCB( Widget, XtPointer ) = 0;
	static void activateCB( Widget, XtPointer, XtPointer );
	virtual void activateCB( Widget, XtPointer ) = 0;
};

typedef UITextLine_c *UITextLine_p;

UITextLine_p UITextLine = (UITextLine_p) NULL;



void UITextLine_c::create (Widget parent, char *widget_name)
{
	Widget children[1];      /* Children to manage */
	Arg al[64];                    /* Arg List */
	register int ac = 0;           /* Arg Count */

	if ( !widget_name )
		widget_name = "UITextLine";

	XtSetArg(al[ac], XmNautoUnmanage, FALSE); ac++;
	UITextLine = XmCreateForm ( parent, widget_name, al, ac );
	ac = 0;
	_xd_rootwidget = UITextLine;
	XtSetArg(al[ac], XmNcolumns, 40); ac++;
	text_ = XmCreateTextField ( UITextLine, "text_", al, ac );
	ac = 0;

	XtSetArg(al[ac], XmNtopAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNleftAttachment, XmATTACH_FORM); ac++;
	XtSetValues ( text_,al, ac );
	ac = 0;
	XtAddCallback (text_, XmNvalueChangedCallback, changedCB, (XtPointer) this);
	XtAddCallback (text_, XmNactivateCallback, activateCB, (XtPointer) this);
	children[ac++] = text_;
	XtManageChildren(children, ac);
	ac = 0;
}

void UITextLine_c::changedCB( Widget widget, XtPointer client_data, XtPointer call_data )
{
	UITextLine_p instance = (UITextLine_p) client_data;
	instance->changedCB ( widget, call_data );
}

void UITextLine_c::activateCB( Widget widget, XtPointer client_data, XtPointer call_data )
{
	UITextLine_p instance = (UITextLine_p) client_data;
	instance->activateCB ( widget, call_data );
}

