/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

// File ExternalEditor.h
// Baudouin Raoult - ECMWF Jul 99

#ifndef ExternalEditor_H
#define ExternalEditor_H

#ifndef Editor_H
#include "Editor.h"
#endif

#ifndef Task_H
#include "Task.h"
#endif

#ifndef TaskObserver_H
#include "TaskObserver.h"
#endif

class ExternalEditor : public Editor, public TaskObserver {
public:

// -- Exceptions
	// None

// -- Contructors

	ExternalEditor(const IconClass&,const string&);

// -- Destructor

	virtual ~ExternalEditor(); // Change to virtual if base class

// -- Convertors
	// None

// -- Operators
	// None

// -- Methods
	// None

// -- Overridden methods
	// None

// -- Class members
	// None

// -- Class methods
	// None

	// Uncomment for persistent, remove otherwise
	// static os_typespec* get_os_typespec();

protected:

// -- Members
	// None

// -- Methods
	
	// void print(ostream&) const; // Change to virtual if base class	

// -- Overridden methods
	// None

// -- Class members
	TaskH task_;
	// None

// -- Class methods
	// None

private:

// No copy allowed

	ExternalEditor(const ExternalEditor&);
	ExternalEditor& operator=(const ExternalEditor&);

// -- Members
	
//	TaskH task_;

// -- Methods

	virtual string command() = 0;

// -- Overridden methods

	// From Editor

	virtual void edit();
	virtual void empty();

	// From TaskObserver

	virtual void success(Task*,const Request&);
	virtual void failure(Task*);


// -- Class members
	// None

// -- Class methods
	// None

// -- Friends

	//friend ostream& operator<<(ostream& s,const ExternalEditor& p)
	//	{ p.print(s); return s; }

};

inline void destroy(ExternalEditor**) {}

// If persistent, uncomment, otherwise remove
//#ifdef _ODI_OSSG_
//OS_MARK_SCHEMA_TYPE(ExternalEditor);
//#endif

#endif
