/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

// File BoolLine.h
// Baudouin Raoult - ECMWF Jul 99

#ifndef BoolLine_H
#define BoolLine_H

#include "UIBoolLine.h"
#include "RequestPanelLine.h"

class BoolLine : public XRequestPanelLine<UIBoolLine_c> {
public:

// -- Contructors

	BoolLine(RequestPanel&,const Parameter&);

// -- Destructor

	virtual ~BoolLine(); 

private:

// No copy allowed

	BoolLine(const BoolLine&);
	BoolLine& operator=(const BoolLine&);

// -- Overridden methods

	// From XD
	virtual void activateCB(Widget,XtPointer);

	// From RequetPanelLine

	virtual void init(Widget);

	// Form RequestPanelItem

	virtual void refresh(const vector<string>&);

};

#endif
