TIROS-N HIRS Radiances ; 1
NOAA 6 HIRS Radiances ; 2
NOAA 7 HIRS Radiances ; 3
NOAA 8 HIRS Radiances ; 4
NOAA 9 HIRS Radiances ; 5
NOAA 10 HIRS Radiances ; 6
NOAA 11 HIRS Radiances ; 7
NOAA 12 HIRS Radiances ; 8
NOAA 14 HIRS Radiances ; 9
NOAA 15 HIRS Radiances ; 10
NOAA 16 HIRS Radiances ; 11
NOAA 17 HIRS Radiances ; 12
NOAA 18 HIRS Radiances ; 13
NOAA 19 HIRS Radiances ; 14
METOP-A HIRS Radiances ; 15
NOAA 8 HIRS Radiances ; 16
NOAA 9 HIRS Radiances ; 17
NOAA 10 HIRS Radiances ; 18
NOAA 11 HIRS Radiances ; 19
NOAA 6 HIRS Radiances ; 20
NOAA 7 HIRS Radiances ; 21
TIROS-N HIRS Radiances ; 22
NOAA 12 HIRS Radiances ; 23
NOAA 14 HIRS Radiances ; 24
METOP-B HIRS Radiances ; 25
NOAA 15 AMSUA Radiances ; 1001
NOAA 16 AMSUA Radiances ; 1002
NOAA 17 AMSUA Radiances ; 1003
NOAA 18 AMSUA Radiances ; 1004
NOAA 19 AMSUA Radiances ; 1005
METOP-A AMSUA Radiances ; 1007
AQUA AMSUA Radiances ; 1008
METOP-B AMSUA Radiances ; 1009
NOAA 15 AMSUB Radiances ; 2001
NOAA 16 AMSUB Radiances ; 2002
NOAA 17 AMSUB Radiances ; 2003
NOAA 18 AMSUB Radiances ; 2004
NOAA 19 MHS Radiances ; 3001
METOP-A MHS Radiances ; 3002
NOAA 18 MHS Radiances ; 3003
METOP-B MHS Radiances ; 3004
GOES 8 GEOS Radiances ; 4002
GOES 9 GEOS Radiances ; 4003
GOES 10 GEOS Radiances ; 4004
GOES 11 GEOS Radiances ; 4005
GOES 12 GEOS Radiances ; 4006
METEOSAT 7 GEOS Radiances ; 4007
METEOSAT 8 GEOS Radiances ; 4008
METEOSAT 9 GEOS Radiances ; 4009
MTSAT-1R GEOS Radiances ; 4010
GOES 13 GEOS Radiances ; 4011
MTSAT-2 GEOS Radiances ; 4012
METEOSAT 9 GEOS Allsky Radiances ; 4013
GOES 14 GEOS Radiances ; 4014
GOES 15 GEOS Radiances ; 4015
METEOSAT 10 GEOS Allsky Radiances ; 4016
METEOSAT 5 GEOS Radiances ; 4017
METEOSAT 8 GEOS Allsky Radiances ; 4018
COMS-1 GEOS Radiances ; 4019
METEOSAT 4 GEOS Radiances ; 4020
METEOSAT 6 GEOS Radiances ; 4021
Himawari 8 GEOS radiances ; 4022
METEOSAT 11 GEOS Allsky Radiances ; 4023
Himawari 9 GEOS radiances ; 4024
ERS 2 GOME O3 ; 5001
METEOSAT 8 SEVIRI O3 ; 5002
METEOSAT 9 SEVIRI O3 ; 5003
AURA MLS O3 ; 5004
AURA OMI O3 ; 5005
NOAA 9 SBUV-2 O3 ; 5006
NOAA 11 SBUV-2 O3 ; 5007
NOAA 14 SBUV-2 O3 ; 5008
NOAA 16 SBUV-2 O3 ; 5009
NOAA 17 SBUV-2 O3 ; 5010
NOAA 18 SBUV-2 O3 ; 5011
NOAA 19 SBUV-2 O3 ; 5012
METOP-A GOME-2 O3 ; 5013
ENVISAT SCIAMACHY O3 ; 5014
ENVISAT GOMOS O3 ; 5015
ENVISAT MIPAS O3 ; 5016
METEOR 3-5 TOMS O3 ; 5017
NIMBUS 7 SBUV-2 O3 ; 5018
AQUA MODIS Aerosol ; 5019
TERRA MODIS Aerosol ; 5020
GOSAT TANSO-FTS Layer volumetric mixing ratio ; 5021
NIMBUS 7 TOMS O3 ; 5022
EARTH PROBE TOMS O3 ; 5023
METOP-A IASI Layer integrated mass density ; 5024
ERS 2 GOME MACC O3 ; 5025
METEOSAT 8 SEVIRI MACC O3 ; 5026
METEOSAT 9 SEVIRI MACC O3 ; 5027
AURA MLS MACC O3 ; 5028
AURA OMI MACC O3 ; 5029
NOAA 9 SBUV-2 MACC O3 ; 5030
NOAA 11 SBUV-2 MACC O3 ; 5031
NOAA 14 SBUV-2 MACC O3 ; 5032
NOAA 16 SBUV-2 MACC O3 ; 5033
NOAA 17 SBUV-2 MACC O3 ; 5034
NOAA 18 SBUV-2 MACC O3 ; 5035
NOAA 19 SBUV-2 MACC O3 ; 5036
METOP-A GOME-2 MACC O3 ; 5037
ENVISAT SCIAMACHY MACC O3 ; 5038
ENVISAT GOMOS MACC O3 ; 5039
ENVISAT MIPAS MACC O3 ; 5040
METEOR 3-5 TOMS MACC O3 ; 5041
NIMBUS 7 SBUV-2 MACC O3 ; 5042
NIMBUS 7 TOMS MACC O3 ; 5043
EARTH PROBE TOMS MACC O3 ; 5044
ENVISAT SCIAMACHY Layer volumetric mixing ratio ; 5045
ENVISAT SCIAMACHY Layer integrated mass density ; 5046
METOP-A GOME-2 Layer integrated mass density ; 5047
AURA OMI Layer integrated mass density ; 5048
TERRA MOPITT Layer integrated mass density ; 5049
ERS-2 GOME Layer integrated mass density ; 5050
NOAA 15 ATOVS ; 5051
NOAA 16 ATOVS ; 5052
NOAA 17 ATOVS ; 5053
NOAA 18 ATOVS ; 5054
NOAA 19 ATOVS ; 5055
METOP-A ATOVS ; 5056
METOP-B GOME-2 O3 ; 5057
METEOSAT 10 SEVIRI O3 ; 5058
NPP OMPS TCO ; 5059
NPP OMPS Nadir Profile ; 5060
METOP-B GOME-2 Layer integrated mass density ; 5061
METOP-B GOME-2 MACC O3 ; 5062
METEOSAT-9 SEVIRI aerosol optical depth ; 5063
METEOSAT-10 SEVIRI aerosol optical depth ; 5064
NPP VIIRS aerosol optical thickness ; 5065
CALIPSO attenuated aerosol lidar backscatter ; 5066
ENVISAT AATSR Aerosol ; 5067
NPP OMPS O3 ; 5068
TOMS ADEOS-I ; 5069
NIMBUS 4 BUV ; 5070
METOP-A GOME-2 PMAP ; 5071
METOP-B GOME-2 PMAP ; 5072
SCISAT-1 ACE-FTS MACC O3 ; 5073
ODIN OSIRIS MACC O3 ; 5074
ODIN SMR MACC O3 ; 5075
SCISAT-1 ACE-FTS O3 ; 5076
ODIN OSIRIS O3 ; 5077
ODIN SMR O3 ; 5078
Sentinel 5P TROPOMI O3 ; 5079
ENVISAT MERIS TCWV ; 6001
METOP-A GPSRO ; 7001
CHAMP GPSRO ; 7002
GRACE A GPSRO ; 7003
COSMIC-1 GPSRO ; 7004
COSMIC-2 GPSRO ; 7005
COSMIC-3 GPSRO ; 7006
COSMIC-4 GPSRO ; 7007
COSMIC-5 GPSRO ; 7008
COSMIC-6 GPSRO ; 7009
TerraSAR-X GPSRO ; 7010
CNOFS GPSRO ; 7011
SAC-C GPSRO ; 7012
METOP-B GPSRO ; 7013
GRACE B GPSRO ; 7014
TanDEM-X GPSRO ; 7015
FY-3C GPSRO ; 7016
Megha-Tropiques GPSRO ; 7017
METEOSAT 2 AMV ; 8001
METEOSAT 3 AMV ; 8002
METEOSAT 4 AMV ; 8003
METEOSAT 5 AMV ; 8004
METEOSAT 6 AMV ; 8005
METEOSAT 7 AMV ; 8006
METEOSAT 8 AMV ; 8007
METEOSAT 9 AMV ; 8008
GMS 5 AMV ; 8009
MTSAT-1R AMV ; 8010
GOES 9 WV AMV ; 8011
GOES 10 AMV ; 8012
GOES 11 AMV ; 8013
GOES 12 AMV ; 8014
NOAA 15 AVHRR IR AMV ; 8015
NOAA 16 AVHRR IR AMV ; 8016
NOAA 17 AVHRR IR AMV ; 8017
NOAA 18 AVHRR IR AMV ; 8018
NOAA 19 AVHRR IR AMV ; 8019
TERRA MODIS AMV ; 8020
AQUA MODIS AMV ; 8021
FY-2C IR AMV ; 8022
FY-2D IR AMV ; 8023
FY-2E IR AMV ; 8024
GOES 13 AMV ; 8025
MTSAT-2 AMV ; 8026
METOP-A AMV ; 8027
GOES 14 AMV ; 8028
GOES 15 AMV ; 8029
METOP-B AMV ; 8030
METEOSAT 10 AMV ; 8031
GMS-3 AMV ; 8032
GMS-4 AMV ; 8033
Meteosat-1 AMV ; 8034
NOAA 7 AVHRR AMV ; 8035
NOAA 9 AVHRR AMV ; 8036
NOAA 10 AVHRR AMV ; 8037
NOAA 11 AVHRR AMV ; 8038
NOAA 12 AVHRR AMV ; 8039
NOAA 14 AVHRR AMV ; 8040
GMS-2 SATOB-AMV ; 8041
GOES 8 AMV ; 8042
GMS-3 SATOB-AMV ; 8043
GMS-1 AMV ; 8044
COMS-1 AMV ; 8045
KALPANA-1 AMV ; 8046
Leo-Geo AMV ; 8047
Dual-Metop AMVs ; 8048
INSAT-3D AMVs ; 8049
NPP AMV ; 8050
GOES 6 AMV subtype 82 ; 8051
GOES 6 AMV subtype 83 ; 8052
Himawari 8 AMV ; 8053
FY-2G AMVs ; 8054
MISR AMV ; 8055
METEOSAT 11 AMV ; 8056
GOES 16 AMV ; 8057
Old ERS 1 Scatterometer ; 9001
New ERS 1 Scatterometer ; 9002
ERS 2 Scatterometer ; 9003
QuickSCAT SeaWind ; 9004
METOP-A ASCAT ; 9005
OceanSat-2 Scatterometer ; 9006
METOP-B ASCAT ; 9007
ISS RapidScat ; 9008
HY-2A HSCAT ; 9009
DMSP 7 SSMI All-sky ; 10001
DMSP 8 SSMI All-sky ; 10002
DMSP 9 SSMI All-sky ; 10003
DMSP 10 SSMI All-sky ; 10004
DMSP 11 SSMI All-sky ; 10005
DMSP 13 SSMI All-sky ; 10006
DMSP 14 SSMI All-sky ; 10007
DMSP 15 SSMI All-sky ; 10008
DMSP 8 SSMI CMSAF All-sky ; 10009
DMSP 10 SSMI CMSAF All-sky ; 10010
DMSP 11 SSMI CMSAF All-sky ; 10011
DMSP 13 SSMI CMSAF All-sky ; 10012
DMSP 14 SSMI CMSAF All-sky ; 10013
DMSP 15 SSMI CMSAF All-sky ; 10014
METOP-A IASI Radiances ; 11001
METOP-B IASI Radiances ; 11002
AQUA AIRS Radiances ; 12001
DMSP 16 SSMIS Radiances All-sky ; 13001
DMSP 17 SSMIS Radiances All-sky ; 13002
DMSP 18 SSMIS Radiances All-sky ; 13003
DMSP 19 SSMIS Radiances All-sky ; 13004
TRMM TMI Radiances All-sky ; 14001
AQUA AMSRE Radiances All-sky ; 15001
Automatic Land SYNOP ; 16001
Manual Land SYNOP ; 16002
Abbreviated SYNOP ; 16003
METAR ; 16004
DRIBU ; 16005
Automatic SHIP ; 16006
Reduced SHIP ; 16007
SHIP ; 16008
Abbreviated SHIP ; 16009
DRIBU-BATHY ; 16010
DRIBU-TESAC ; 16011
Ground-Based GPS ; 16012
Land PILOT ; 16013
PILOT SHIP ; 16014
American Wind Profiler 1 ; 16015
American Wind Profiler 2 ; 16016
European Wind Profiler ; 16017
Japanese Wind Profiler ; 16018
TEMP SHIP ; 16019
DROP Sonde ; 16020
Mobile TEMP ; 16021
Land TEMP ; 16022
ROCOB TEMP ; 16023
SHIP ROCOB TEMP ; 16024
Additional Land Surface Data ; 16025
AIREP ; 16026
CODAR ; 16027
COLBA ; 16028
AMDAR ; 16029
ACARS ; 16030
PAOB ; 16031
PAOB via Sat code ; 16032
SATEM 500km Low Level Temperature ; 16037
SATEM 500km PWC ; 16038
SATEM 500km High Level Temperature ; 16039
SATEM 500km Merged ; 16040
SATEM 250km Low Level Temperature ; 16041
SATEM 250km PWC ; 16042
SATEM 250km High Level Temperature ; 16043
SATEM 250km Merged ; 16044
BUFR LAND TEMP ; 16045
BUFR SHIP TEMP ; 16046
Land High Resolution TEMP ; 16047
SHIP High Resolution TEMP ; 16048
Ocean Station Vessel On Station ; 16049
Ocean Station Vessel Off Station ; 16050
Station Or Ship On Ice ; 16051
Ocean Bottle And Low Resolution Conductivity Temperature Depth CTD And XCTD ; 16052
Mechanical Or Digital Or Micro Bathythermograph MBT ; 16053
Expandable Bathythermograph XBT ; 16054
Coastal-Marine Automated Network CMAN ; 16055
Coastal Or Island Station ; 16056
Fixed Ocean Platform Or Rig ; 16057
Mobile land SYNOP ; 16058
Forecaster bogus ; 16059
Tropical cyclone bogus ; 16060
High Resolution Conductivity Temperature Depth CTD And XCTD ; 16061
Undulating Oceanographic Recorder UOR ; 16062
Autonomous Pinniped Bathythermograph APBT ; 16063
Underwater Ocean Glider ; 16064
Automatic METAR ; 16065
TAMDAR ; 16066
ACARs with mixing ratio ; 16067
BUFR LAND PILOT ; 16068
BUFR SHIP PILOT ; 16069
BUFR LAND SYNOP ; 16070
BUFR LAND SYNOP 2 ; 16071
BUFR LAND SYNOP WMO Region 6 ; 16072
BUFR LAND SYNOP 1 HOURLY ; 16073
BUFR SHIP SYNOP ; 16074
Mobile PILOT ; 16075
BUFR LAND SYNOP ; 16076
Roadside observations ; 16077
Surface WOW observations ; 16078
SFERICS lightning observations ; 16079
ATDNET lightning observations ; 16080
Upper air WISDOM ; 16081
WIGOS AMDAR ; 16082
BUFR MOORED BUOYS ; 16083
BUFR DRIFTING BUOYS ; 16084
Ceilometer ground lidar ; 16085
SMOS ; 18001
CORIOLIS WINDSAT All-sky ; 19001
DMSP 8 SSMI ; 20001
DMSP 9 SSMI ; 20002
DMSP 10 SSMI ; 20003
DMSP 11 SSMI ; 20004
DMSP 13 SSMI ; 20005
DMSP 14 SSMI ; 20006
DMSP 15 SSMI ; 20007
NOAA 15 AMSUA Radiances All-sky ; 21001
NOAA 16 AMSUA Radiances All-sky ; 21002
NOAA 17 AMSUA Radiances All-sky ; 21003
NOAA 18 AMSUA Radiances All-sky ; 21004
NOAA 19 AMSUA Radiances All-sky ; 21005
METOP-A AMSUA Radiances All-sky ; 21007
AQUA AMSUA Radiances All-sky ; 21008
METOP-B AMSUA Radiances All-sky ; 21009
AQUA AMSRE Radiances ; 22001
TRMM TMI Radiances ; 23001
DMSP 16 SSMIS Radiances ; 24001
DMSP 17 SSMIS Radiances ; 24002
DMSP 18 SSMIS Radiances ; 24003
DMSP 19 SSMIS Radiances ; 24004
Ground-Based Radar ; 25001
FY-3A MWHS Radiances ; 26001
FY-3B MWHS Radiances ; 26002
FY-3A MWTS Radiances ; 27001
FY-3B MWTS Radiances ; 27002
FY-3A MWRI Radiances All-sky ; 28001
FY-3B MWRI Radiances All-sky ; 28002
FY-3C MWRI Radiances All-sky ; 28003
FY-3A IRAS Radiances ; 29001
FY-3B IRAS Radiances ; 29002
FY-3C IRAS Radiances ; 29003
TIROS-N MSU Radiances ; 30001
NOAA 6 MSU Radiances ; 30002
NOAA 7 MSU Radiances ; 30003
NOAA 8 MSU Radiances ; 30004
NOAA 9 MSU Radiances ; 30005
NOAA 10 MSU Radiances ; 30006
NOAA 11 MSU Radiances ; 30007
NOAA 12 MSU Radiances ; 30008
NOAA 14 MSU Radiances ; 30009
NOAA 8 MSU TOVS1B Radiances ; 30010
NOAA 10 MSU TOVS1B Radiances ; 30011
NOAA 11 MSU TOVS1B Radiances ; 30012
NOAA 6 MSU TOVS1B Radiances ; 30013
NOAA 7 MSU TOVS1B Radiances ; 30014
TIROS-N MSU TOVS1B Radiances ; 30015
NOAA 14 MSU TOVS1B Radiances ; 30016
NOAA 9 MSU TOVS1B Radiances ; 30017
NOAA 12 MSU TOVS1B Radiances ; 30018
TIROS-N SSU Radiances ; 31001
NOAA 6 SSU Radiances ; 31002
NOAA 7 SSU Radiances ; 31003
NOAA 8 SSU Radiances ; 31004
NOAA 9 SSU Radiances ; 31005
NOAA 11 SSU Radiances ; 31006
NOAA 12 SSU Radiances ; 31007
NOAA 14 SSU Radiances ; 31008
NOAA 8 SSU TOVS1B Radiances ; 31009
NOAA 9 SSU TOVS1B Radiances ; 31010
NOAA 11 SSU TOVS1B Radiances ; 31011
NOAA 6 SSU TOVS1B Radiances ; 31012
NOAA 7 SSU TOVS1B Radiances ; 31013
TIROS-N SSU TOVS1B Radiances ; 31014
NOAA 14 SSU TOVS1B Radiances ; 31015
NOAA 12 SSU TOVS1B Radiances ; 31016
NOAA 2 VTPR1 Radiances ; 32001
NOAA 3 VTPR1 Radiances ; 32002
NOAA 4 VTPR1 Radiances ; 32003
NOAA 5 VTPR1 Radiances ; 32004
NOAA 2 VTPR2 Radiances ; 33001
NOAA 3 VTPR2 Radiances ; 33002
NOAA 4 VTPR2 Radiances ; 33003
NOAA 5 VTPR2 Radiances ; 33004
NPP ATMS Radiances ; 34001
TERRA MOPITT layer integrated mass density with averaging kernels and a priori profile ; 35001
ERS-2 GOME layer integrated mass density with averaging kernels ; 35002
ENVISAT SCIAMACHY layer integrated mass density with averaging kernels ; 35003
METOP-A GOME-2 layer integrated mass density with averaging kernels ; 35004
METOP-A IASI layer volumetric mixing ratio with averaging kernels and a priori profile ; 35005
ENVISAT SCIAMACHY layer volumetric mixing ratio with averaging kernels and a priori profile ; 35006
METOP-A IASI layer integrated mass density with averaging kernels and a priori profile ; 35007
AURA OMI layer integrated mass density with averaging kernels ; 35008
GOSAT TANSO-FTS layer volumetric mixing ratio with averaging kernels and a priori profile ; 35009
METOP-A GOME-2 layer integrated mass density with averaging kernels and a priori profile ; 35010
METOP-B IASI layer integrated mass density with averaging kernels and a priori profile ; 35011
METOP-B GOME-2 layer integrated mass density with averaging kernels ; 35012
METOP-B GOME-2 layer integrated mass density with averaging kernels and a priori profile ; 35013
Sentinel 5P TROPOMI layer volumetric mixing ratio with averaging kernels and a priori profile ; 35014
Sentinel 5P TROPOMI layer integrated mass density with averaging kernels and a priori profile ; 35015
Sentinel 5P TROPOMI layer integrated mass density with averaging kernels ; 35016
METOP-B IASI layer volumetric mixing ratio with averaging kernels and a priori profile ; 35017
NPP CRIS Radiances ; 36001
ENVISAT RALT WAVE ; 37001
JASON 1 RALT WAVE ; 37002
JASON 2 RALT WAVE ; 37003
ERS 1 RALT WAVE ; 37004
ERS 2 RALT WAVE ; 37005
CRYOSAT 2 RALT WAVE ; 37006
SARAL RALT WAVE ; 37007
JASON 3 RALT WAVE ; 37008
Sentinel 3A RALT WAVE ; 37009
1h rain gauge accumulation ; 39001
3h rain gauge accumulation ; 39002
6h rain gauge accumulation ; 39003
12h rain gauge accumulation ; 39004
24h rain gauge accumulation ; 39005
NESDIS IMS ; 40001
NESDIS SST ; 40002
MeteoFrance SST ; 40003
GCOM-W1 AMSR-2 Radiances All-sky ; 41001
SAPHIR All-sky ; 42001
AMSUB NOAA 15 AMSUB Radiances All-sky ; 43001
AMSUB NOAA 16 AMSUB Radiances All-sky ; 43002
AMSUB NOAA 17 AMSUB Radiances All-sky ; 43003
AMSUB NOAA 18 AMSUB Radiances All-sky ; 43004
MHS NOAA 19 MHS Radiances All-sky ; 44001
MHS METOP-A MHS Radiances All-sky ; 44002
MHS NOAA 18 MHS Radiances All-sky ; 44003
MHS METOP-B MHS Radiances All-sky ; 44004
AEOLUS HLOS Wind Level 2B ; 45001
IRIS Nimbus-3 Radiances ; 46001
IRIS Nimbus-4 Radiances ; 46002
ENVISAT AATSR brightness teperatures ; 48001
NPP ATMS Radiances All-sky ; 49001
GPM GMI Radiances All-sky ; 50001
METOP-B GODAE sea surface temperature ; 51001
Meteosat 9 GODAE sea surface temperature ; 51002
NOAA 18 GODAE sea surface temperature ; 51003
NOAA 19 GODAE sea surface temperature ; 51004
GOES 13 GODAE sea surface temperature ; 51005
TRMM GODAE sea surface temperature ; 51006
ENVISAT GODAE sea surface temperature ; 51007
MSG3 GODAE sea surface temperature ; 51008
UKMO ATOVS ; 52001
European surface air quality BUFR data ; 53001
UKPP derived cloud data ; 54001
FY-3C MWTS2 Radiances ; 55001
DMSP 8 SSMI 1DVAR TCWV Cloudy-Sky ; 56001
DMSP 10 SSMI 1DVAR TCWV Cloudy-Sky ; 56002
DMSP 11 SSMI 1DVAR TCWV Cloudy-Sky ; 56003
DMSP 13 SSMI 1DVAR TCWV Cloudy-Sky ; 56004
DMSP 14 SSMI 1DVAR TCWV Cloudy-Sky ; 56005
DMSP 15 SSMI 1DVAR TCWV Cloudy-Sky ; 56006
FY-3C MWHS2 Radiances All-sky ; 57001
DMSP 11 SSMT2 Radiances ; 58001
DMSP 12 SSMT2 Radiances ; 58002
DMSP 14 SSMT2 Radiances ; 58003
DMSP 15 SSMT2 Radiances ; 58004
SMAP ; 59001
UKMO TOVS ; 60001
TEST REPORTYPE 1 ; 98001
TEST REPORTYPE 2 ; 98002
TEST REPORTYPE 3 ; 98003
TEST REPORTYPE 4 ; 98004
TEST REPORTYPE 5 ; 98005
TEST REPORTYPE 6 ; 98006
TEST REPORTYPE 7 ; 98007
TEST REPORTYPE 8 ; 98008
TEST REPORTYPE 9 ; 98009
TEST REPORTYPE 10 ; 98010
