/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

// File Timeout.h
// Baudouin Raoult - ECMWF Oct 98

#ifndef Timeout_H
#define Timeout_H

#include <Xm/Xm.h>


class Timeout {
public:

// -- Exceptions
	// None

// -- Contructors

	Timeout(double);

// -- Destructor

	virtual ~Timeout(); // Change to virtual if base class

// -- Convertors
	// None

// -- Operators
	// None

// -- Methods
	// None

	void enable();
	void disable();
	void frequency(double);
	void drift(double,double);

	virtual void run() = 0;

// -- Overridden methods
	// None

// -- Class members
	// None

// -- Class methods
	// None

	// Uncomment for persistent, remove otherwise
	// static os_typespec* get_os_typespec();

protected:

// -- Members


// -- Methods
	
	// void print(ostream&) const; // Change to virtual if base class	

// -- Overridden methods
	// None

// -- Class members
	// None

// -- Class methods
	// None

private:

// No copy allowed

	Timeout(const Timeout&);
	Timeout& operator=(const Timeout&);

// -- Members

	Boolean actived_;
	XtIntervalId id_;
	double frequency_;
	int running_;

// -- Methods
	// None

// -- Overridden methods
	// None

// -- Class members

// -- Class methods
	// None
	static void timeoutCB(XtPointer,XtIntervalId*);

// -- Friends

	//friend ostream& operator<<(ostream& s,const Timeout& p)
	//	{ p.print(s); return s; }

};


#endif
