/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

/*
** Generated by X-Designer
*/
#ifndef _UILayoutDrawer_h
#define _UILayoutDrawer_h

#define XD_MOTIF

#include <xdclass.h>

class UILayoutDrawer_c: public xd_XmForm_c {
public:
	virtual void create (Widget parent, char *widget_name = NULL);
protected:
	Widget UILayoutDrawer;
	Widget split1_;
	Widget split2_;
	Widget add_;
	Widget delete_;
	Widget join_;
	Widget group_;
	Widget ungroup_;
public:
	static void deleteSelectionCB( Widget, XtPointer, XtPointer );
	virtual void deleteSelectionCB( Widget, XtPointer ) = 0;
	static void addPageCB( Widget, XtPointer, XtPointer );
	virtual void addPageCB( Widget, XtPointer ) = 0;
	static void splitVerCB( Widget, XtPointer, XtPointer );
	virtual void splitVerCB( Widget, XtPointer ) = 0;
	static void splitHorCB( Widget, XtPointer, XtPointer );
	virtual void splitHorCB( Widget, XtPointer ) = 0;
	static void ungroupSelectionCB( Widget, XtPointer, XtPointer );
	virtual void ungroupSelectionCB( Widget, XtPointer ) = 0;
	static void groupSelectionCB( Widget, XtPointer, XtPointer );
	virtual void groupSelectionCB( Widget, XtPointer ) = 0;
	static void joinSelectionCB( Widget, XtPointer, XtPointer );
	virtual void joinSelectionCB( Widget, XtPointer ) = 0;
};

typedef UILayoutDrawer_c *UILayoutDrawer_p;


extern UILayoutDrawer_p UILayoutDrawer;


#endif
