/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

// File ShellService.h
// Baudouin Raoult - ECMWF Jul 99

#ifndef ShellService_H
#define ShellService_H

#ifndef Service_H
#include "Service.h"
#endif


class ShellService : public Service {
public:

// -- Exceptions
	// None

// -- Contructors

	ShellService(const string&);

// -- Destructor

	virtual ~ShellService(); // Change to virtual if base class

// -- Convertors
	// None

// -- Operators
	// None

// -- Methods
	// None

// -- Overridden methods
	// None

// -- Class members
	// None

// -- Class methods
	// None

	// Uncomment for persistent, remove otherwise
	// static os_typespec* get_os_typespec();

protected:

// -- Members
	// None

// -- Methods
	
	// void print(ostream&) const; // Change to virtual if base class	

// -- Overridden methods
	// None

// -- Class members
	// None

// -- Class methods
	// None

private:

// No copy allowed

	ShellService(const ShellService&);
	ShellService& operator=(const ShellService&);

// -- Members
	// None

// -- Methods
	// None

// -- Overridden methods

	virtual Task* task(const Action&,IconObject*);

// -- Class members
	// None

// -- Class methods
	// None

// -- Friends

	//friend ostream& operator<<(ostream& s,const ShellService& p)
	//	{ p.print(s); return s; }

};

inline void destroy(ShellService**) {}

// If persistent, uncomment, otherwise remove
//#ifdef _ODI_OSSG_
//OS_MARK_SCHEMA_TYPE(ShellService);
//#endif

#endif
