/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/


#ifndef  COLORS_H
#define  COLORS_H

extern WidgetClass colorsWidgetClass;
typedef struct _ColorsClassRec * ColorsWidgetClass;
typedef struct _ColorsRec      * ColorsWidget;

#if defined(__cplusplus) || defined(c_plusplus)
extern "C" {
#endif

extern Widget CreateColorsWidget(Widget,char*,Arg*,int);
extern void ColorsSetPixels(Widget,int,Pixel*);

#if defined(__cplusplus) || defined(c_plusplus)
}
#endif


#endif
