/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

// File ColorName.h
// Baudouin Raoult - ECMWF Oct 99

#ifndef ColorName_H
#define ColorName_H

#ifndef UIColorName_H
#include "UIColorName.h"
#endif

#include <string>
using std::string;


struct request;
class ColourEditor;

class ColorName : public UIColorName_c  {
public:

// -- Exceptions
	// None

// -- Contructors

	ColorName(ColourEditor&,const string&);

// -- Destructor

	~ColorName(); // Change to virtual if base class

// -- Convertors
	// None

// -- Operators
	// None

// -- Methods

	void set();
	void unset();

// -- Overridden methods
	// None

// -- Class members
	// None

// -- Class methods

	static void build(Widget);
	static void load(request*);

	// Uncomment for persistent, remove otherwise
	// static os_typespec* get_os_typespec();

protected:

// -- Members
	// None

// -- Methods
	
	// void print(ostream&) const; // Change to virtual if base class	

// -- Overridden methods
	// None

// -- Class members
	// None

// -- Class methods
	// None

private:

// No copy allowed

	ColorName(const ColorName&);
	ColorName& operator=(const ColorName&);

// -- Members

	ColourEditor& owner_;
	string magics_;

// -- Methods
	// None

// -- Overridden methods
	// None

	virtual void activateCB(Widget,XtPointer);

// -- Class members
	// None

// -- Class methods
	// None

// -- Friends

	//friend ostream& operator<<(ostream& s,const ColorName& p)
	//	{ p.print(s); return s; }

};

inline void destroy(ColorName**) {}

// If persistent, uncomment, otherwise remove
//#ifdef _ODI_OSSG_
//OS_MARK_SCHEMA_TYPE(ColorName);
//#endif

#endif
