/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

// File StationaryDrawers.h
// Baudouin Raoult - ECMWF Oct 99

#ifndef StationaryDrawers_H
#define StationaryDrawers_H

#ifndef FolderDrawers_H
#include "FolderDrawers.h"
#endif

struct request;

class StationaryDrawers : public FolderDrawers {
public:

// -- Exceptions
	// None

// -- Contructors

	StationaryDrawers();

// -- Destructor

	~StationaryDrawers(); // Change to virtual if base class

// -- Convertors
	// None

// -- Operators
	// None

// -- Methods
	// None

// -- Overridden methods
	// None

// -- Class members
	// None

// -- Class methods

	static void load(request*);

	// Uncomment for persistent, remove otherwise
	// static os_typespec* get_os_typespec();

protected:

// -- Members
	// None

// -- Methods
	
	// void print(ostream&) const; // Change to virtual if base class	

// -- Overridden methods
	// None

// -- Class members
	// None

// -- Class methods
	// None

private:

// No copy allowed

	StationaryDrawers(const StationaryDrawers&);
	StationaryDrawers& operator=(const StationaryDrawers&);

// -- Members
	// None

// -- Methods
	// None

// -- Overridden methods

	// From FolderDrawers

	virtual bool fill(int);

// -- Class members
	// None

// -- Class methods
	// None

// -- Friends

	//friend ostream& operator<<(ostream& s,const StationaryDrawers& p)
	//	{ p.print(s); return s; }

};

inline void destroy(StationaryDrawers**) {}

// If persistent, uncomment, otherwise remove
//#ifdef _ODI_OSSG_
//OS_MARK_SCHEMA_TYPE(StationaryDrawers);
//#endif

#endif
