/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

// File ExternalTextEditor.h
// Baudouin Raoult - ECMWF Jul 99

#ifndef ExternalTextEditor_H
#define ExternalTextEditor_H

#ifndef ExternalEditor_H
#include "ExternalEditor.h"
#endif


class ExternalTextEditor : public ExternalEditor {
public:

// -- Exceptions
	// None

// -- Contructors

	ExternalTextEditor(const IconClass&,const string&);

// -- Destructor

	virtual ~ExternalTextEditor(); // Change to virtual if base class

// -- Convertors
	// None

// -- Operators
	// None

// -- Methods
	// None

// -- Overridden methods
	// None

// -- Class members
	// None

// -- Class methods
	// None

	// Uncomment for persistent, remove otherwise
	// static os_typespec* get_os_typespec();

protected:

// -- Members
	// None

// -- Methods
	
	// void print(ostream&) const; // Change to virtual if base class	

// -- Overridden methods
	// None

// -- Class members
	// None

// -- Class methods
	// None

private:

// No copy allowed

	ExternalTextEditor(const ExternalTextEditor&);
	ExternalTextEditor& operator=(const ExternalTextEditor&);

// -- Members


// -- Methods
	// None

// -- Overridden methods

	// From ExternalEditor
	virtual string command();

// -- Class members
	// None

// -- Class methods
	// None

// -- Friends

	//friend ostream& operator<<(ostream& s,const ExternalTextEditor& p)
	//	{ p.print(s); return s; }

};

inline void destroy(ExternalTextEditor**) {}

// If persistent, uncomment, otherwise remove
//#ifdef _ODI_OSSG_
//OS_MARK_SCHEMA_TYPE(ExternalTextEditor);
//#endif

#endif
