/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

/*
** Generated by X-Designer
*/
#ifndef _UIRequestPanel_h
#define _UIRequestPanel_h

#define XD_MOTIF

#include "xdclass.h"

class UIRequestPanel_c: public xd_XmForm_c {
public:
	virtual void create (Widget parent, char *widget_name = NULL);
protected:
	Widget UIRequestPanel;
	Widget rowcol_;
};

typedef UIRequestPanel_c *UIRequestPanel_p;


extern UIRequestPanel_p UIRequestPanel;



#endif
