/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

// File OutputDrawer.h
// Baudouin Raoult - ECMWF Jul 99

#ifndef OutputDrawer_H
#define OutputDrawer_H

#ifndef EditorDrawer_H
#include "EditorDrawer.h"
#endif

#include "UIOutputDrawer.h"

class MacroEditor;

class OutputDrawer : public XEditorDrawer<UIOutputDrawer_c> {
public:

// -- Exceptions
	// None

// -- Contructors

	OutputDrawer(MacroEditor&);

// -- Destructor

	virtual ~OutputDrawer(); // Change to virtual if base class

// -- Convertors
	// None

// -- Operators
	// None

// -- Methods

// -- Overridden methods
	// None

// -- Class members
	// None

// -- Class methods
	// None

	// Uncomment for persistent, remove otherwise
	// static os_typespec* get_os_typespec();

protected:

// -- Members
	// None

// -- Methods
	
	// void print(ostream&) const; // Change to virtual if base class	

// -- Overridden methods
	// None

// -- Class members
	// None

// -- Class methods
	// None

private:

// No copy allowed

	OutputDrawer(const OutputDrawer&);
	OutputDrawer& operator=(const OutputDrawer&);

// -- Members

	MacroEditor& editor_;

// -- Methods
	// None

// -- Overridden methods

	virtual void clickCB(Widget,XtPointer);

// -- Class members
	// None

// -- Class methods
	// None

// -- Friends

	//friend ostream& operator<<(ostream& s,const OutputDrawer& p)
	//	{ p.print(s); return s; }

};

inline void destroy(OutputDrawer**) {}

// If persistent, uncomment, otherwise remove
//#ifdef _ODI_OSSG_
//OS_MARK_SCHEMA_TYPE(OutputDrawer);
//#endif

#endif
