/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/


/*
** Generated by X-Designer 
*/

extern Widget run_top;
extern Widget run_form;
extern Widget run_text;

extern Widget top_widget;
extern Widget form_widget;
extern Widget text;
extern Widget file_menu;
extern Widget help_button;
extern Widget macro_menu;
extern Widget record_menu;
extern Widget edit_menu;

