/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

/*
** Generated by X-Designer
*/
#ifndef _UIFolderWindow_h
#define _UIFolderWindow_h

#define XD_MOTIF

#include <xdclass.h>

class UIFolderWindow_c: public xd_ApplicationShell_c {
public:
	virtual void create (Display *display, char *app_name, int app_argc, char **app_argv, char *app_class_name = NULL);
protected:
	Widget UIFolderWindow;
	Widget form_;
	Widget drag_;
	Widget commands_;
	Widget menu_;
	Widget drawer_;
	Widget close_;
	Widget tools_;
	Widget recent_;
	Widget top_;
	Widget up_;
	Widget defaults_;
	Widget help_;
public:
	static void helpMenuCB( Widget, XtPointer, XtPointer );
	virtual void helpMenuCB( Widget, XtPointer ) = 0;
	static void messagesCB( Widget, XtPointer, XtPointer );
	virtual void messagesCB( Widget, XtPointer ) = 0;
	static void rescanCB( Widget, XtPointer, XtPointer );
	virtual void rescanCB( Widget, XtPointer ) = 0;
	static void toggleSizeCB( Widget, XtPointer, XtPointer );
	virtual void toggleSizeCB( Widget, XtPointer ) = 0;
	static void prefCB( Widget, XtPointer, XtPointer );
	virtual void prefCB( Widget, XtPointer ) = 0;
	static void newLinkCB( Widget, XtPointer, XtPointer );
	virtual void newLinkCB( Widget, XtPointer ) = 0;
	static void newIconCB( Widget, XtPointer, XtPointer );
	virtual void newIconCB( Widget, XtPointer ) = 0;
	static void recentCB( Widget, XtPointer, XtPointer );
	virtual void recentCB( Widget, XtPointer ) = 0;
	static void showLastCB( Widget, XtPointer, XtPointer );
	virtual void showLastCB( Widget, XtPointer ) = 0;
	static void sortByClassCB( Widget, XtPointer, XtPointer );
	virtual void sortByClassCB( Widget, XtPointer ) = 0;
	static void sortByNameCB( Widget, XtPointer, XtPointer );
	virtual void sortByNameCB( Widget, XtPointer ) = 0;
	static void fileMapCB( Widget, XtPointer, XtPointer );
	virtual void fileMapCB( Widget, XtPointer ) = 0;
	static void commandCB( Widget, XtPointer, XtPointer );
	virtual void commandCB( Widget, XtPointer ) = 0;
	static void exitCB( Widget, XtPointer, XtPointer );
	virtual void exitCB( Widget, XtPointer ) = 0;
	static void closeCB( Widget, XtPointer, XtPointer );
	virtual void closeCB( Widget, XtPointer ) = 0;
};

typedef UIFolderWindow_c *UIFolderWindow_p;


extern UIFolderWindow_p UIFolderWindow;


#endif
