/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

// File ActionCommand.h
// Baudouin Raoult - ECMWF Jul 99

#ifndef ActionCommand_H
#define ActionCommand_H


#ifndef Command_H
#include "Command.h"
#endif


class ActionCommand : public Command {
public:

// -- Exceptions
	// None

// -- Contructors

	ActionCommand(const string& name);

// -- Destructor

	virtual ~ActionCommand(); // Change to virtual if base class

// -- Convertors
	// None

// -- Operators
	// None

// -- Methods

	virtual void execute(IconObject*);

// -- Overridden methods
	// None

// -- Class members
	// None

// -- Class methods

	static void execute(const string&,IconObject*);

protected:

// -- Members


// -- Methods
	
	// void print(ostream&) const; // Change to virtual if base class	

// -- Overridden methods
	// None

// -- Class members
	// None

// -- Class methods
	// None

private:

// No copy allowed

	ActionCommand(const ActionCommand&);
	ActionCommand& operator=(const ActionCommand&);

// -- Members
	// None

// -- Methods
	// None

// -- Overridden methods
	// None

// -- Class members
	// None

// -- Class methods
	// None

// -- Friends

	//friend ostream& operator<<(ostream& s,const ActionCommand& p)
	//	{ p.print(s); return s; }

};

#endif
