/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

// File XMLabel.h
// Baudouin Raoult - ECMWF Jul 99

#ifndef XMLabel_H
#define XMLabel_H

#include <string>
using std::string;

#include <Xm/Xm.h>


class XMLabel {
public:

// -- Exceptions
	// None

// -- Contructors

	XMLabel(Widget);

// -- Destructor

	~XMLabel(); // Change to virtual if base class

// -- Convertors
	// None

// -- Operators
	// None

// -- Methods

	void set(const string&);

// -- Overridden methods
	// None

// -- Class members
	// None

// -- Class methods
	// None

	// Uncomment for persistent, remove otherwise
	// static os_typespec* get_os_typespec();

protected:

// -- Members
	// None

// -- Methods
	
	// void print(ostream&) const; // Change to virtual if base class	

// -- Overridden methods
	// None

// -- Class members
	// None

// -- Class methods
	// None

private:

// No copy allowed

	XMLabel(const XMLabel&);
	XMLabel& operator=(const XMLabel&);

// -- Members
	
	Widget label_;

// -- Methods
	// None

// -- Overridden methods
	// None

// -- Class members
	// None

// -- Class methods
	// None

// -- Friends

	//friend ostream& operator<<(ostream& s,const XMLabel& p)
	//	{ p.print(s); return s; }

};

inline void destroy(XMLabel**) {}

// If persistent, uncomment, otherwise remove
//#ifdef _ODI_OSSG_
//OS_MARK_SCHEMA_TYPE(XMLabel);
//#endif

#endif
