/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

#ifndef uPlotBatchApp_H
#define uPlotBatchApp_H

#include <MvRequest.h>
#include <MvProtocol.h>
#include <MvApplication.h>

using namespace std;

class uPlotBatchApp : public MvApplication
{
public:
	// Constructor
	uPlotBatchApp (int& ac, char** av, const char* name=NULL);

	// Destructor
	virtual ~uPlotBatchApp();

	// Processing the request
	virtual void serve (MvProtocol&,MvRequest&);

private:
	// No copy allowed
	uPlotBatchApp (const uPlotBatchApp&);
	uPlotBatchApp& operator=(const uPlotBatchApp&){return *this;}
};

#endif
