/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

#ifndef LINE_H
#define LINE_H

#define XtNsizes   "sizes"
#define XtCSizes   "Sizes"

extern WidgetClass  lineWidgetClass;

typedef struct _LineClassRec *LineWidgetClass;
typedef struct _LineRec      *LineWidget;
#ifdef _NO_PROTO
 
extern Widget CreateLine();
 
#else
 
#if defined(__cplusplus) || defined(c_plusplus)
extern "C" {
#endif
 
Widget    CreateLine(Widget,String,Arg*,int);
void LineAdjust(Widget);
#if defined(__cplusplus) || defined(c_plusplus)
}
#endif
 
#endif /* _NO_PROTO */
 


#endif /* LINE_H */
