/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

// File CatalogDrawer.h
// Baudouin Raoult - ECMWF Jul 99

#ifndef CatalogDrawer_H
#define CatalogDrawer_H

#ifndef EditorDrawer_H
#include "EditorDrawer.h"
#endif

#include "UICatalogDrawer.h"

class RetrieveEditor;

class CatalogDrawer : public XEditorDrawer<UICatalogDrawer_c> {
public:

// -- Exceptions
	// None

// -- Contructors

	CatalogDrawer(RetrieveEditor&);

// -- Destructor

	virtual ~CatalogDrawer(); // Change to virtual if base class

// -- Convertors
	// None

// -- Operators
	// None

// -- Methods

// -- Overridden methods
	// None

// -- Class members
	// None

// -- Class methods
	// None

	// Uncomment for persistent, remove otherwise
	// static os_typespec* get_os_typespec();

protected:

// -- Members
	// None

// -- Methods
	
	// void print(ostream&) const; // Change to virtual if base class	

// -- Overridden methods
	// None

// -- Class members
	// None

// -- Class methods
	// None

private:

// No copy allowed

	CatalogDrawer(const CatalogDrawer&);
	CatalogDrawer& operator=(const CatalogDrawer&);

// -- Members

	RetrieveEditor& editor_;

// -- Methods
	// None

// -- Overridden methods

	virtual void homeCB(Widget,XtPointer);
	virtual void helpCB(Widget,XtPointer);
	virtual void topCatalogCB(Widget,XtPointer);
	virtual void inCatalogCB(Widget,XtPointer);
	virtual void grabCatalogCB(Widget,XtPointer);

// -- Class members
	// None

// -- Class methods
	// None

// -- Friends

	//friend ostream& operator<<(ostream& s,const CatalogDrawer& p)
	//	{ p.print(s); return s; }

};

inline void destroy(CatalogDrawer**) {}

// If persistent, uncomment, otherwise remove
//#ifdef _ODI_OSSG_
//OS_MARK_SCHEMA_TYPE(CatalogDrawer);
//#endif

#endif
