/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

// File Unknown.h
// Baudouin Raoult - ECMWF Jul 99

#ifndef Unknown_H
#define Unknown_H

#ifndef FileObject_H
#include "FileObject.h"
#endif


#include <string>
using std::string;


class Unknown : public FileObject {
public:

// -- Exceptions
	// None

// -- Contructors

	Unknown(Folder* parent,
		const IconClass& kind, const string& name,
		IconInfo* info);

// -- Destructor

	virtual ~Unknown(); // Change to virtual if base class

// -- Convertors
	// None

// -- Operators
	// None

// -- Methods


// -- Overridden methods
	// None

// -- Class members
	// None

// -- Class methods
	// None

	// Uncomment for persistent, remove otherwise
	// static os_typespec* get_os_typespec();

protected:

// -- Members
	// None

// -- Methods
	
	// void print(ostream&) const; // Change to virtual if base class	

// -- Overridden methods
	// None

// -- Class members
	// None

// -- Class methods
	// None

private:

// No copy allowed

	Unknown(const Unknown&);
	Unknown& operator=(const Unknown&);

// -- Members

// -- Methods
	// None

// -- Overridden methods

	// From IconObject

	virtual set<string> can();
	virtual void createFiles();

// -- Class members
	// None

// -- Class methods
	// None

// -- Friends

	//friend ostream& operator<<(ostream& s,const Unknown& p)
	//	{ p.print(s); return s; }

};

inline void destroy(Unknown**) {}

// If persistent, uncomment, otherwise remove
//#ifdef _ODI_OSSG_
//OS_MARK_SCHEMA_TYPE(Unknown);
//#endif

#endif
