/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

// File TeeBuffer.h
// Baudouin Raoult - ECMWF Jun 00

#ifndef TeeBuffer_H
#define TeeBuffer_H

#include "inc_iostream.h"

class TeeBuffer : public streambuf {
public:

// -- Exceptions
	// None

// -- Contructors

	TeeBuffer(ostream&,ostream&);

// -- Destructor

	~TeeBuffer(); // Change to virtual if base class

// -- Convertors
	// None

// -- Operators
	// None

// -- Methods
	// None

// -- Overridden methods
	// None

// -- Class members
	// None

// -- Class methods
	// None

	// Uncomment for persistent, remove otherwise
	// static os_typespec* get_os_typespec();

protected:

// -- Members
	// None

// -- Methods
	

// -- Overridden methods

	int   overflow(int);
	int   sync();

// -- Class members
	// None

// -- Class methods
	// None

private:

// No copy allowed

	TeeBuffer(const TeeBuffer&);
	TeeBuffer& operator=(const TeeBuffer&);

// -- Members

	char buffer_[1024];
	ostream& one_;
	ostream& two_;

// -- Methods

	void dumpBuffer(void);

// -- Overridden methods
	// None

// -- Class members
	// None

// -- Class methods
	// None

// -- Friends

	//friend ostream& operator<<(ostream& s,const TeeBuffer& p)
	//	{ p.print(s); return s; }

};

inline void destroy(TeeBuffer**) {}

// If persistent, uncomment, otherwise remove
//#ifdef _ODI_OSSG_
//OS_MARK_SCHEMA_TYPE(TeeBuffer);
//#endif

#endif
