#ifndef uPLOTMANAGER_H
#define uPLOTMANAGER_H

/**************
  Application uPlotManager

  Manager the uPlot visualisation module
***************/

#include "Metview.h"

class uPlotManager : public MvService {

public:

	// Constructor
	uPlotManager(const char* kw);

	// Destructor
	~uPlotManager() {}

	void serve(MvRequest&,MvRequest&);

protected:


};

#endif
