/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

/*
** Generated by X-Designer
*/
#ifndef _UILogWindow_h
#define _UILogWindow_h

#define XD_MOTIF

#include <xdclass.h>

class UILogWindow_c: public xd_ApplicationShell_c {
public:
	virtual void create (Display *display, char *app_name, int app_argc, char **app_argv, char *app_class_name = NULL);
protected:
	Widget UILogWindow;
	Widget form_;
	Widget text_;
public:
	static void freezeCB( Widget, XtPointer, XtPointer );
	virtual void freezeCB( Widget, XtPointer ) = 0;
	static void clearCB( Widget, XtPointer, XtPointer );
	virtual void clearCB( Widget, XtPointer ) = 0;
	static void closeCB( Widget, XtPointer, XtPointer );
	virtual void closeCB( Widget, XtPointer ) = 0;
};

typedef UILogWindow_c *UILogWindow_p;


extern UILogWindow_p UILogWindow;


#endif
