/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

/*
** Generated by X-Designer
*/
#ifndef _UIGridDrawer_h
#define _UIGridDrawer_h

#define XD_MOTIF

#include <xdclass.h>

class UIGridDrawer_c: public xd_XmForm_c {
public:
	virtual void create (Widget parent, char *widget_name = NULL);
protected:
	Widget UIGridDrawer;
	Widget x_;
	Widget y_;
public:
	static void yEntryCB( Widget, XtPointer, XtPointer );
	virtual void yEntryCB( Widget, XtPointer ) = 0;
	static void xEntryCB( Widget, XtPointer, XtPointer );
	virtual void xEntryCB( Widget, XtPointer ) = 0;
	static void snapCB( Widget, XtPointer, XtPointer );
	virtual void snapCB( Widget, XtPointer ) = 0;
	static void showCB( Widget, XtPointer, XtPointer );
	virtual void showCB( Widget, XtPointer ) = 0;
	static void setCB( Widget, XtPointer, XtPointer );
	virtual void setCB( Widget, XtPointer ) = 0;
};

typedef UIGridDrawer_c *UIGridDrawer_p;


extern UIGridDrawer_p UIGridDrawer;


#endif
